import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import od_cpu_dma_latency
from insights.parsers.od_cpu_dma_latency import OdCpuDmaLatency
from insights.tests import context_wrap

CONTENT_OD_CPU_DMA_LATENCY = """
  2000000000
"""

CONTENT_OD_CPU_DMA_LATENCY_EMPTY = ""


def test_doc_examples():
    env = {'cpu_dma_latency': OdCpuDmaLatency(context_wrap(CONTENT_OD_CPU_DMA_LATENCY))}
    failed, total = doctest.testmod(od_cpu_dma_latency, globs=env)
    assert failed == 0


def test_OdCpuDmaLatency():
    d = OdCpuDmaLatency(context_wrap(CONTENT_OD_CPU_DMA_LATENCY))
    assert d.force_latency == 2000000000

    with pytest.raises(SkipComponent):
        OdCpuDmaLatency(context_wrap(CONTENT_OD_CPU_DMA_LATENCY_EMPTY))



"""
Rules for data collection
"""
from __future__ import absolute_import

import json
import logging
import os
import six
import stat
import yaml

from six.moves import configparser as ConfigParser

from insights.client.constants import InsightsConstants as constants

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)
NETWORK = constants.custom_network_log_level


def correct_format(parsed_data, expected_keys, filename):
    '''
    Ensure the parsed file matches the needed format
    Returns True, <message> on error
    Returns False, None on success
    '''
    # validate keys are what we expect
    def is_list_of_strings(data):
        '''
        Helper function for correct_format()
        '''
        if data is None:
            # nonetype, no data to parse. treat as empty list
            return True
        if not isinstance(data, list):
            return False
        for l in data:
            if not isinstance(l, six.string_types):
                return False
        return True

    keys = parsed_data.keys()
    invalid_keys = set(keys).difference(expected_keys)
    if invalid_keys:
        return True, ('Unknown section(s) in %s: ' % filename + ', '.join(invalid_keys) +
                      '\nValid sections are ' + ', '.join(expected_keys) + '.')

    # validate format (lists of strings)
    for k in expected_keys:
        if k in parsed_data:
            if k == 'patterns' and isinstance(parsed_data['patterns'], dict):
                if 'regex' not in parsed_data['patterns']:
                    return True, 'Patterns section contains an object but the "regex" key was not specified.'
                if 'regex' in parsed_data['patterns'] and len(parsed_data['patterns']) > 1:
                    return True, 'Unknown keys in the patterns section. Only "regex" is valid.'
                if not is_list_of_strings(parsed_data['patterns']['regex']):
                    return True, 'regex section under patterns must be a list of strings.'
                continue
            if not is_list_of_strings(parsed_data[k]):
                return True, '%s section must be a list of strings.' % k
    return False, None


def load_yaml(filename):
    try:
        with open(filename) as f:
            loaded_yaml = yaml.safe_load(f)
        if loaded_yaml is None:
            logger.debug('%s is empty.', filename)
            return {}
    except (yaml.YAMLError, yaml.parser.ParserError) as e:
        # can't parse yaml from conf
        raise RuntimeError('ERROR: Cannot parse %s.\n'
                           'If using any YAML tokens such as [] in an expression, '
                           'be sure to wrap the expression in quotation marks.\n\nError details:\n%s\n' % (filename, e))
    if not isinstance(loaded_yaml, dict):
        # loaded data should be a dict with at least one key
        raise RuntimeError('ERROR: Invalid YAML loaded.')
    return loaded_yaml


def verify_permissions(f):
    '''
    Verify 600 permissions on a file
    '''
    mode = stat.S_IMODE(os.stat(f).st_mode)
    if not mode == 0o600:
        raise RuntimeError("Invalid permissions on %s. "
                           "Expected 0600 got %s" % (f, oct(mode)))
    logger.debug("Correct file permissions on %s", f)


class InsightsUploadConf(object):
    """
    Insights spec configuration
    """

    def __init__(self, config, conn=None):
        """
        Load configuration from parent
        """
        self.config = config
        self.remove_file = config.remove_file
        self.redaction_file = config.redaction_file
        self.content_redaction_file = config.content_redaction_file
        self.tags_file = config.tags_file

        # set rm_conf as a class attribute so we can observe it
        #   in create_report
        self.rm_conf = None

        # attribute to set when using file-redaction.yaml instead of
        #   remove.conf, for reporting purposes. True by default
        #   since new format is favored.
        self.using_new_format = True

    def get_rm_conf_old(self):
        """
        Get excluded files config from remove_file.
        """
        # Convert config object into dict
        self.using_new_format = False
        parsedconfig = ConfigParser.RawConfigParser()
        if not self.remove_file:
            # no filename defined, return nothing
            logger.debug('remove_file is undefined')
            return None
        if not os.path.isfile(self.remove_file):
            logger.debug('%s not found. No data files, commands,'
                         ' or patterns will be ignored, and no keyword obfuscation will occur.', self.remove_file)
            return None
        try:
            verify_permissions(self.remove_file)
        except RuntimeError as e:
            if self.config.validate:
                # exit if permissions invalid and using --validate
                raise RuntimeError('ERROR: %s' % e)
            logger.warning('WARNING: %s', e)
        try:
            parsedconfig.read(self.remove_file)
            sections = parsedconfig.sections()

            if not sections:
                # file has no sections, skip it
                logger.debug('Remove.conf exists but no parameters have been defined.')
                return None

            if sections != ['remove']:
                raise RuntimeError('ERROR: invalid section(s) in remove.conf. Only "remove" is valid.')

            expected_keys = ('commands', 'files', 'patterns', 'keywords')
            rm_conf = {'new_format': False}
            for item, value in parsedconfig.items('remove'):
                if item not in expected_keys:
                    raise RuntimeError('ERROR: Unknown key in remove.conf: ' + item +
                                       '\nValid keys are ' + ', '.join(expected_keys) + '.')
                if six.PY3:
                    rm_conf[item] = [v.strip() for v in value.strip().encode('utf-8').decode('unicode-escape').split(',')]
                else:
                    rm_conf[item] = [v.strip() for v in value.strip().decode('string-escape').split(',')]
            self.rm_conf = rm_conf
        except ConfigParser.Error as e:
            # can't parse config file at all
            logger.debug(e)
            logger.debug('To configure using YAML, please use file-redaction.yaml and file-content-redaction.yaml.')
            raise RuntimeError('ERROR: Cannot parse the remove.conf file.\n'
                               'See %s for more information.' % self.config.logging_file)
        logger.warning('WARNING: remove.conf is deprecated. Please use file-redaction.yaml and file-content-redaction.yaml. See https://access.redhat.com/articles/4511681 for details.')
        return self.rm_conf

    def load_redaction_file(self, fname):
        '''
        Load the YAML-style file-redaction.yaml
            or file-content-redaction.yaml files
        '''
        if fname not in (self.redaction_file, self.content_redaction_file):
            # invalid function use, should never get here in a production situation
            return None
        if not fname:
            # no filename defined, return nothing
            logger.debug('redaction_file or content_redaction_file is undefined')
            return None
        if not fname or not os.path.isfile(fname):
            if fname == self.redaction_file:
                logger.debug('%s not found. No files or commands will be skipped.', self.redaction_file)
            elif fname == self.content_redaction_file:
                logger.debug('%s not found. '
                             'No patterns will be skipped and no keyword obfuscation will occur.', self.content_redaction_file)
            return None
        try:
            verify_permissions(fname)
        except RuntimeError as e:
            if self.config.validate:
                # exit if permissions invalid and using --validate
                raise RuntimeError('ERROR: %s' % e)
            logger.warning('WARNING: %s', e)
        loaded = load_yaml(fname)
        if fname == self.redaction_file:
            err, msg = correct_format(loaded, ('commands', 'files', 'components'), fname)
        elif fname == self.content_redaction_file:
            err, msg = correct_format(loaded, ('patterns', 'keywords'), fname)
        if err:
            # YAML is correct but doesn't match the format we need
            raise RuntimeError('ERROR: ' + msg)
        return loaded

    def get_rm_conf(self):
        '''
        Try to load the the "new" version of
        remove.conf (file-redaction.yaml and file-content-redaction.yaml)
        '''
        rm_conf = {}
        redact_conf = self.load_redaction_file(self.redaction_file)
        content_redact_conf = self.load_redaction_file(self.content_redaction_file)

        if not redact_conf and not content_redact_conf:
            # no file-redaction.yaml or file-content-redaction.yaml defined,
            #   try to use remove.conf
            self.rm_conf = self.get_rm_conf_old()
        else:
            rm_conf.update(new_format=True)
            rm_conf.update(redact_conf or {})
            rm_conf.update(content_redact_conf or {})
            # remove Nones, empty strings, and empty lists
            self.rm_conf = dict((k, v) for k, v in rm_conf.items() if v)

        if self.rm_conf and ('/etc/insights-client/machine-id' in self.rm_conf.get('files', []) or
                'insights.specs.default.DefaultSpecs.machine_id' in self.rm_conf.get('components', [])):
            logger.warning("WARNING: Spec machine_id will be skipped for redaction; as it would cause issues, please remove it from %s.", self.redaction_file)
        # return the RAW rm_conf
        return self.rm_conf

    def get_tags_conf(self):
        '''
        Try to load the tags.conf file
        '''
        if not os.path.isfile(self.tags_file):
            logger.info("%s does not exist", self.tags_file)
            return None
        else:
            try:
                load_yaml(self.tags_file)
                logger.info("%s loaded successfully", self.tags_file)
            except RuntimeError:
                logger.warning("Invalid YAML. Unable to load %s", self.tags_file)
                return None

    def validate(self):
        '''
        Validate remove.conf and tags.conf
        '''
        self.get_tags_conf()
        success = self.get_rm_conf()
        if not success:
            logger.info('No contents in the blacklist configuration to validate.')
            return None
        # Using print here as this could contain sensitive information
        print('Blacklist configuration parsed contents:')
        print(json.dumps(success, indent=4))
        logger.info('Parsed successfully.')
        return True

    def create_report(self):
        def length(lst):
            '''
            Because of how the INI remove.conf is parsed,
            an empty value in the conf will produce
            the value [''] when parsed. Do not include
            these in the report
            '''
            if len(lst) == 1 and lst[0] == '':
                return 0
            return len(lst)

        num_commands = 0
        num_files = 0
        num_components = 0
        num_patterns = 0
        num_keywords = 0
        using_regex = False

        if self.rm_conf:
            for key in self.rm_conf:
                if key == 'commands':
                    num_commands = length(self.rm_conf['commands'])
                if key == 'files':
                    num_files = length(self.rm_conf['files'])
                if key == 'components':
                    num_components = length(self.rm_conf['components'])
                if key == 'patterns':
                    if isinstance(self.rm_conf['patterns'], dict):
                        num_patterns = length(self.rm_conf['patterns']['regex'])
                        using_regex = True
                    else:
                        num_patterns = length(self.rm_conf['patterns'])
                if key == 'keywords':
                    num_keywords = length(self.rm_conf['keywords'])

        return {
            'obfuscate': self.config.obfuscate,
            'obfuscate_hostname': self.config.obfuscate_hostname,
            'commands': num_commands,
            'files': num_files,
            'components': num_components,
            'patterns': num_patterns,
            'keywords': num_keywords,
            'using_new_format': self.using_new_format,
            'using_patterns_regex': using_regex
        }



from insights.parsers.nscd_conf import NscdConf, NscdConfLine
from insights.tests import context_wrap

NSCD_CONF = """
#
# /etc/nscd.conf
#
# An example Name Service Cache config file.  This file is needed by nscd.
#
# Legal entries are:
#
#       logfile                 <file>
#       debug-level             <level>
#       threads                 <initial #threads to use>
#       max-threads             <maximum #threads to use>
#       server-user             <user to run server as instead of root>
#               server-user is ignored if nscd is started with -S parameters
#       stat-user               <user who is allowed to request statistics>
#       reload-count            unlimited|<number>
#       paranoia                <yes|no>
#       restart-interval        <time in seconds>
#
#       enable-cache            <service> <yes|no>
#       positive-time-to-live   <service> <time in seconds>
#       negative-time-to-live   <service> <time in seconds>
#       suggested-size          <service> <prime number>
#       check-files             <service> <yes|no>
#       persistent              <service> <yes|no>
#       shared                  <service> <yes|no>
#       max-db-size             <service> <number bytes>
#       auto-propagate          <service> <yes|no>
#
# Currently supported cache names (services): passwd, group, hosts, services
#


#       logfile                 /var/log/nscd.log
#       threads                 4
#       max-threads             32
        server-user             nscd
#       stat-user               somebody
        debug-level             0
#       reload-count            5
        paranoia                no
#       restart-interval        3600

        enable-cache            passwd          no
        positive-time-to-live   passwd          600
        negative-time-to-live   passwd          20
        suggested-size          passwd          211
        check-files             passwd          yes
        persistent              passwd          yes
        shared                  passwd          yes
        max-db-size             passwd          33554432
        auto-propagate          passwd          yes

        enable-cache            group           no
        positive-time-to-live   group           3600
        negative-time-to-live   group           60
        suggested-size          group           211
        check-files             group           yes
        persistent              group           yes
        shared                  group           yes
        max-db-size             group           33554432
        auto-propagate          group           yes

        enable-cache            hosts           yes
        positive-time-to-live   hosts           3600
        negative-time-to-live   hosts           20
        suggested-size          hosts           211
        check-files             hosts           yes
        persistent              hosts           yes
        shared                  hosts           yes
        max-db-size             hosts           33554432

        enable-cache            services        yes
        positive-time-to-live   services        28800
        negative-time-to-live   services        20
        suggested-size          services        211
        check-files             services        yes
        persistent              services        yes
        shared                  services        yes
        max-db-size             services        33554432

        enable-cache            netgroup        no
        positive-time-to-live   netgroup        28800
        negative-time-to-live   netgroup        20
        suggested-size          netgroup        211
        check-files             netgroup        yes
        persistent              netgroup        yes
        shared                  netgroup        yes
        max-db-size             netgroup        33554432

"""


def test_nscd_conf():
    conf = NscdConf(context_wrap(NSCD_CONF))
    assert conf is not None
    lines = [l for l in conf]
    assert len(lines) == 45
    assert len(conf.data) == 45
    assert lines[0] == conf.data[0]

    # Service Attributes tests
    assert conf.service_attributes("bad-name") == []
    assert conf.service_attributes("hosts") == [
        NscdConfLine("enable-cache", "hosts", "yes"),
        NscdConfLine("positive-time-to-live", "hosts", "3600"),
        NscdConfLine("negative-time-to-live", "hosts", "20"),
        NscdConfLine("suggested-size", "hosts", "211"),
        NscdConfLine("check-files", "hosts", "yes"),
        NscdConfLine("persistent", "hosts", "yes"),
        NscdConfLine("shared", "hosts", "yes"),
        NscdConfLine("max-db-size", "hosts", "33554432")
    ]

    # Attribute tests
    assert conf.attribute("server-user") == "nscd"
    assert conf.attribute("restart-interval") is None

    # Filter tests
    assert conf.filter("server-user") == [
        NscdConfLine(attribute="server-user", service=None, value="nscd")
    ]
    assert conf.filter("log-file") == []
    assert conf.filter("enable-cache", "netgroup") == [
        NscdConfLine(attribute="enable-cache", service="netgroup", value="no")
    ]
    assert conf.filter("enable-cache") == [
        NscdConfLine(attribute="enable-cache", service="passwd", value="no"),
        NscdConfLine(attribute="enable-cache", service="group", value="no"),
        NscdConfLine(attribute="enable-cache", service="hosts", value="yes"),
        NscdConfLine(attribute="enable-cache", service="services", value="yes"),
        NscdConfLine(attribute="enable-cache", service="netgroup", value="no")
    ]
    assert conf.filter("cache") == [
        NscdConfLine(attribute="enable-cache", service="passwd", value="no"),
        NscdConfLine(attribute="enable-cache", service="group", value="no"),
        NscdConfLine(attribute="enable-cache", service="hosts", value="yes"),
        NscdConfLine(attribute="enable-cache", service="services", value="yes"),
        NscdConfLine(attribute="enable-cache", service="netgroup", value="no")
    ]



# -*- coding: utf-8 -*-
import os

from mock.mock import patch
from pytest import mark

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.core.spec_cleaner import Cleaner


@mark.parametrize(("line", "expected"), [
    ("test_no_ip", "test_no_ip"),
    ("test 127.0.0.1", "test 127.0.0.1"),
    ("radius_ip_1=10.0.0.1", "radius_ip_1=10.230.230.1"),
    (
        (
            "        inet 10.0.2.15"
            "  netmask 255.255.255.0"
            "  broadcast 10.0.2.255"
        ),
        (
            "        inet 10.230.230.3"
            "  netmask 10.230.230.1"
            "  broadcast 10.230.230.2"
        ),
    ),
    (
        "radius_ip_1=10.0.0.100-10.0.0.200",
        "radius_ip_1=10.230.230.1-10.230.230.2",
    ),
])
def test_obfuscate_ip_match(line, expected):
    c = InsightsConfig(obfuscate=True)
    pp = Cleaner(c, {})
    actual = pp._obfuscate_line(line, [pp._sub_ip])
    assert actual == expected


@mark.parametrize(("line", "expected"), [
    (
        (
            "        inet 10.0.2.155"
            "  netmask 10.0.2.1"
            "  broadcast 10.0.2.15"
        ),
        (
            "        inet 10.230.230.1"
            "  netmask 10.230.230.3"
            "  broadcast 10.230.230.2"
        ),
    ),
])
def test_obfuscate_ip_match_IP_overlap(line, expected):
    c = InsightsConfig(obfuscate=True)
    pp = Cleaner(c, {})
    actual = pp._obfuscate_line(line, [pp._sub_ip])
    assert actual == expected


@mark.parametrize(("line", "expected"), [
    ("test_no_ip", "test_no_ip"),
    ("test 127.0.0.1", "test 127.0.0.1"),
    (
        "tcp6       0      0 10.0.0.1:23           10.0.0.110:63564   ESTABLISHED 0",
        "tcp6       0      0 10.230.230.2:23       10.230.230.1:63564 ESTABLISHED 0"
    ),
    (
        "tcp6  10.0.0.11    0 10.0.0.1:23       10.0.0.111:63564    ESTABLISHED 0",
        "tcp6  10.230.230.2 0 10.230.230.3:23   10.230.230.1:63564  ESTABLISHED 0"
    ),
    (
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         172.31.0.1\n",
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         10.230.230.1\n"
    ),
    (
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         172.31.111.11\n",
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         10.230.230.1 \n"
    ),
])
def test_obfuscate_ip_match_IP_overlap_netstat(line, expected):
    c = InsightsConfig(obfuscate=True)
    pp = Cleaner(c, {})
    actual1 = pp._obfuscate_line(line, [pp._sub_ip_netstat])
    actual2 = pp._obfuscate_line(line, [pp._sub_ip_netstat])
    assert actual1 == expected
    assert actual2 == expected


@mark.parametrize(("original", "expected"), [
    (
        "{\"name\":\"shadow-utils\","
        "\"epoch\":\"2\","
        "\"version\":\"4.1.5.1\","
        "\"release\":\"5.el6\","
        "\"arch\":\"x86_64\","
        "\"installtime\":\"Wed 13 Jan 2021 10:04:18 AM CET\","
        "\"buildtime\":\"1455012203\","
        "\"vendor\":\"Red Hat, Inc.\","
        "\"buildhost\":\"x86-027.build.eng.bos.redhat.com\","
        "\"sigpgp\":"
        "\"RSA/8, "
        "Tue 08 Mar 2016 11:15:08 AM CET, "
        "Key ID 199e2f91fd431d51\"}",

        "{\"name\":\"shadow-utils\","
        "\"epoch\":\"2\","
        "\"version\":\"10.230.230.1\","
        "\"release\":\"5.el6\","
        "\"arch\":\"x86_64\","
        "\"installtime\":\"Wed 13 Jan 2021 10:04:18 AM CET\","
        "\"buildtime\":\"1455012203\","
        "\"vendor\":\"Red Hat, Inc.\","
        "\"buildhost\":\"x86-027.build.eng.bos.redhat.com\","
        "\"sigpgp\":"
        "\"RSA/8, "
        "Tue 08 Mar 2016 11:15:08 AM CET, "
        "Key ID 199e2f91fd431d51\"}",
    )
])
@patch("insights.core.spec_cleaner.Cleaner._ip2db", return_value="10.230.230.1")
def test_obfuscate_ip_false_positive(_ip2db, original, expected):
    c = InsightsConfig(obfuscate=True)
    pp = Cleaner(c, {})
    actual = pp._obfuscate_line(original, [pp._sub_ip])
    assert actual == expected
    # BUT works well without "obfuscate=['ip']
    actual = pp._obfuscate_line(original, [])
    assert actual == original


def test_obfuscate_hostname():
    hostname = 'test1.abc.com'
    line = "a line with %s here, test2.abc.com, test.redhat.com" % hostname
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(c, {}, hostname)
    actual = pp._obfuscate_line(line, [pp._sub_hostname])
    assert 'test1' not in actual
    assert 'test2' not in actual
    assert 'abc.com' not in actual
    assert len(actual.split('.')[0].split()[-1]) == 12
    assert '.example.com' in actual

    line = "a line w/o hostname, but test2.abc.com only"
    actual = pp._obfuscate_line(line, [pp._sub_hostname])
    assert 'test2' not in actual
    assert 'abc.com' not in actual
    assert '.example.com' in actual
    assert len(actual.split('.')[0].split()[-1]) != 12

    hostname = 'test1'  # Short hostname
    line = "a line with %s here, test2.def.com" % hostname
    pp = Cleaner(c, {}, hostname)
    actual = pp._obfuscate_line(line, [pp._sub_hostname])
    assert hostname not in actual
    assert 'test2.def.com' in actual

    line = "a line w/o hostname"
    hostname = 'test1.abc.com'
    pp = Cleaner(c, {}, hostname)
    actual = pp._obfuscate_line(line, [pp._sub_hostname])
    assert line == actual


@patch("insights.core.spec_cleaner.determine_hostname", return_value='test1.abc.com')
def test_obfuscate_hostname_determine_hostanme(hn):
    hostname = 'test1.abc.com'
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, display_name='disp.abc.com')
    line = "a line with %s here, test2.def.com" % hostname
    pp = Cleaner(c, {})  # passed empty hostname to cleaner, determain it
    actual = pp._obfuscate_line(line, [pp._sub_hostname])
    assert hostname not in actual
    assert len(actual.split('.')[0].split()[-1]) == 12
    assert 'test2.def.com' in actual


@patch("insights.core.spec_cleaner.determine_hostname", return_value='dt_test.abc.com')
def test_cleaner_fqdn(de_hn):
    fqdn = 'test.abc.com'
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, display_name='disp.abc.com')
    pp = Cleaner(c, {}, fqdn)  # pass fqdn to cleaner
    assert pp.fqdn == fqdn
    assert len(pp.obfuscated_fqdn.split('.')[0]) == 12

    fqdn1 = 'test.def.com'
    pp = Cleaner(c, {}, fqdn1)  # pass fqdn1 to cleaner
    assert pp.fqdn == fqdn1
    assert len(pp.obfuscated_fqdn.split('.')[0]) == 12

    pp = Cleaner(c, {})  # pass None "hostname" to cleaner
    assert pp.fqdn == "dt_test.abc.com"  # get hostname from determine_hostname, but not display_name
    assert len(pp.obfuscated_fqdn.split('.')[0]) == 12


def test_obfuscate_hostname_and_ip():
    hostname = 'test1.abc.com'
    line = "test1.abc.com, 10.0.0.1 test1.abc.loc, 20.1.4.7 smtp.abc.com, 10.1.2.7 lite.abc.com"
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(c, {}, hostname)
    result = pp._obfuscate_line(line, [pp._sub_ip, pp._sub_hostname])
    assert 'example.com' in result
    assert '10.230.230' in result
    for item in line.split():
        assert item not in result


def test_clean_file_obfuscate():
    conf = InsightsConfig(obfuscate=True)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # netstat_-neopa
    line = "tcp6       0      0 10.0.0.1:23           10.0.0.110:63564   ESTABLISHED 0"
    ret = "tcp6       0      0 10.230.230.2:23       10.230.230.1:63564 ESTABLISHED 0"

    test_dir = os.path.join(arch.archive_dir, 'data', 'etc')
    os.makedirs(test_dir)
    pp = Cleaner(conf, {})

    # netstat
    test_file = os.path.join(arch.archive_dir, 'data', 'testfile.netstat_-neopa')
    with open(test_file, 'w') as t:
        t.write(line)
    pp.clean_file(test_file, no_obfuscate=[])
    # file is changed per netstat logic
    with open(test_file, 'r') as t:
        assert ret == ''.join(t.readlines())

    arch.delete_archive_dir()


def test_clean_file_obfuscate_disabled_by_no_obfuscate():
    conf = InsightsConfig(obfuscate=True)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # netstat_-neopa
    line = "tcp6       0      0 10.0.0.1:23           10.0.0.110:63564   ESTABLISHED 0"

    test_dir = os.path.join(arch.archive_dir, 'data', 'etc')
    os.makedirs(test_dir)
    pp = Cleaner(conf, {})

    # netstat
    test_file = os.path.join(arch.archive_dir, 'data', 'testfile.netstat_-neopa')
    with open(test_file, 'w') as t:
        t.write(line)
    pp.clean_file(test_file, no_obfuscate=['ip'])
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert line == ''.join(t.readlines())

    arch.delete_archive_dir()


@patch("insights.core.spec_cleaner.Cleaner._redact_line")
def test_clean_file_non_exist(redact_func):
    conf = InsightsConfig(obfuscate=True)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    test_dir = os.path.join(arch.archive_dir, 'data', 'etc')
    os.makedirs(test_dir)
    pp = Cleaner(conf, {})

    pp.clean_file('non_existing_file', no_obfuscate=[])
    redact_func.assert_not_called()

    # empty file
    test_file = os.path.join(arch.archive_dir, 'data', 'etc', 'x.conf')
    with open(test_file, 'w'):
        pass
    pp.clean_file(test_file, no_obfuscate=[])
    redact_func.assert_not_called()

    arch.delete_archive_dir()



"""
Sysconfig - files in ``/etc/sysconfig/``
========================================

This is a collection of parsers that all deal with the system's configuration
files under the ``/etc/sysconfig/`` folder.  Parsers included in this module
are:

CorosyncSysconfig - file ``/etc/sysconfig/corosync``
----------------------------------------------------

ChronydSysconfig - file ``/etc/sysconfig/chronyd``
--------------------------------------------------

DirsrvSysconfig - file ``/etc/sysconfig/dirsrv``
------------------------------------------------

DockerStorageSetupSysconfig - file ``/etc/sysconfig/docker-storage-setup``
--------------------------------------------------------------------------

DockerSysconfig - file ``/etc/sysconfig/docker``
------------------------------------------------

DockerSysconfigStorage - file ``/etc/sysconfig/docker-storage``
---------------------------------------------------------------

ForemanTasksSysconfig - file ``/etc/sysconfig/foreman-tasks``
-------------------------------------------------------------

HttpdSysconfig - file ``/etc/sysconfig/httpd``
----------------------------------------------

IrqbalanceSysconfig - file ``/etc/sysconfig/irqbalance``
--------------------------------------------------------

KdumpSysconfig - file ``/etc/sysconfig/kdump``
----------------------------------------------

KernelSysconfig - file ``/etc/sysconfig/kernel``
------------------------------------------------

LibvirtGuestsSysconfig - file ``/etc/sysconfig/libvirt-guests``
---------------------------------------------------------------

MemcachedSysconfig - file ``/etc/sysconfig/memcached``
------------------------------------------------------

MongodSysconfig - file ``/etc/sysconfig/mongod``
------------------------------------------------

NetconsoleSysconfig -file ``/etc/sysconfig/netconsole``
-------------------------------------------------------

NetworkSysconfig -file ``/etc/sysconfig/network``
-------------------------------------------------

NfsSysconfig - file ``/etc/sysconfig/nfs``
------------------------------------------

NtpdSysconfig - file ``/etc/sysconfig/ntpd``
--------------------------------------------

PrelinkSysconfig - file ``/etc/sysconfig/prelink``
--------------------------------------------------

SbdSysconfig - file ``/etc/sysconfig/sbd``
------------------------------------------

SshdSysconfig - file ``/etc/sysconfig/sshd``
--------------------------------------------

StonithSysconfig - file ``/etc/sysconfig/stonith``
--------------------------------------------------

PuppetserverSysconfig - file ``/etc/sysconfig/puppetserver``
------------------------------------------------------------

Up2DateSysconfig - file ``/etc/sysconfig/rhn/up2date``
------------------------------------------------------

VirtWhoSysconfig - file ``/etc/sysconfig/virt-who``
---------------------------------------------------

IfCFGStaticRoute - files ``/etc/sysconfig/network-scripts/route-*``
-------------------------------------------------------------------

GrubSysconfig - files ``/etc/sysconfig/grub``
---------------------------------------------

OracleasmSysconfig - files ``/etc/sysconfig/oracleasm``
-------------------------------------------------------

Pcsd - files ``/etc/sysconfig/pcsd``
------------------------------------
"""


from insights import parser, SysconfigOptions, get_active_lines
from insights.specs import Specs


@parser(Specs.corosync)
class CorosyncSysconfig(SysconfigOptions):
    """
    This parser reads the ``/etc/sysconfig/corosync`` file.  It uses the
    ``SysconfigOptions`` parser class to convert the file into a dictionary of
    options.  It also provides the ``options`` property as a helper to retrieve
    the ``COROSYNC_OPTIONS`` variable.

    Sample Input::

        # COROSYNC_INIT_TIMEOUT specifies number of seconds to wait for corosync
        # initialization (default is one minute).
        COROSYNC_INIT_TIMEOUT=60
        # COROSYNC_OPTIONS specifies options passed to corosync command
        # (default is no options).
        # See "man corosync" for detailed descriptions of the options.
        COROSYNC_OPTIONS=""

    Examples:
        >>> 'COROSYNC_OPTIONS' in cs_syscfg
        True
        >>> cs_syscfg.options
        ''
    """
    @property
    def options(self):
        """ (str): The value of the ``COROSYNC_OPTIONS`` variable."""
        return self.data.get('COROSYNC_OPTIONS', '')


@parser(Specs.sysconfig_chronyd)
class ChronydSysconfig(SysconfigOptions):
    """
    This parser analyzes the ``/etc/sysconfig/chronyd`` configuration file.

    Sample Input::

      OPTIONS="-d"
      #HIDE="me"

    Examples:
        >>> 'OPTIONS' in chronyd_syscfg
        True
        >>> 'HIDE' in chronyd_syscfg
        False
        >>> chronyd_syscfg['OPTIONS']
        '-d'

    """
    pass


@parser(Specs.dirsrv)
class DirsrvSysconfig(SysconfigOptions):
    """
    This parser parses the `dirsrv` service's start-up configuration
    ``/etc/sysconfig/dirsrv``.

    Sample Input::

        #STARTPID_TIME=10 ; export STARTPID_TIME
        #PID_TIME=600 ; export PID_TIME
        KRB5CCNAME=/tmp/krb5cc_995
        KRB5_KTNAME=/etc/dirsrv/ds.keytab

    Examples:
        >>> dirsrv_syscfg.get('KRB5_KTNAME')
        '/etc/dirsrv/ds.keytab'
        >>> 'PID_TIME' in dirsrv_syscfg
        False
    """
    pass


@parser(Specs.docker_storage_setup)
class DockerStorageSetupSysconfig(SysconfigOptions):
    """
    Parser for parsing ``/etc/sysconfig/docker-storage-setup``

    Sample Input::

        VG=vgtest
        AUTO_EXTEND_POOL=yes
        ##name = mydomain
        POOL_AUTOEXTEND_THRESHOLD=60
        POOL_AUTOEXTEND_PERCENT=20

    Examples:
        >>> dss_syscfg['VG'] # Pseudo-dict access
        'vgtest'
        >>> 'name' in dss_syscfg
        False
        >>> dss_syscfg.get('POOL_AUTOEXTEND_THRESHOLD')
        '60'

    """
    pass


@parser(Specs.docker_sysconfig)
class DockerSysconfig(SysconfigOptions):
    """
    Parser for parsing the ``/etc/sysconfig/docker`` file using the standard
    ``SysconfigOptions`` parser class.  The 'OPTIONS' variable is also provided
    in the ``options`` property as a convenience.

    Sample Input::

        OPTIONS="--selinux-enabled"
        DOCKER_CERT_PATH="/etc/docker"

    Examples:
        >>> 'OPTIONS' in docker_syscfg
        True
        >>> docker_syscfg['OPTIONS']
        '--selinux-enabled'
        >>> docker_syscfg.options
        '--selinux-enabled'
        >>> docker_syscfg['DOCKER_CERT_PATH']
        '/etc/docker'
    """
    @property
    def options(self):
        """ Return the value of the 'OPTIONS' variable, or '' if not defined. """
        return self.data.get('OPTIONS', '')


@parser(Specs.docker_storage)
class DockerSysconfigStorage(SysconfigOptions):
    """
    A Parser for /etc/sysconfig/docker-storage.

    Sample input::
        DOCKER_STORAGE_OPTIONS="--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/dockervg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true"

    Examples:
        >>> 'DOCKER_STORAGE_OPTIONS' in docker_syscfg_storage
        True
        >>> docker_syscfg_storage["DOCKER_STORAGE_OPTIONS"]
        '--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/dockervg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true'
        >>> docker_syscfg_storage.storage_options
        '--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/dockervg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true'
    """
    @property
    def storage_options(self):
        """ Return the value of the 'DOCKER_STORAGE_OPTIONS' variable, or '' if not defined. """
        return self.data.get('DOCKER_STORAGE_OPTIONS', '')


@parser(Specs.foreman_tasks_config)
class ForemanTasksSysconfig(SysconfigOptions):
    """
    Parse the ``/etc/sysconfig/foreman-tasks`` configuration file.

    Sample configuration file::

        FOREMAN_USER=foreman
        BUNDLER_EXT_HOME=/usr/share/foreman
        RAILS_ENV=production
        FOREMAN_LOGGING=warn

    Examples:
        >>> ft_syscfg['RAILS_ENV']
        'production'
        >>> 'AUTO' in ft_syscfg
        False
    """
    pass


@parser(Specs.sysconfig_httpd)
class HttpdSysconfig(SysconfigOptions):
    """
    This parser analyzes the ``/etc/sysconfig/httpd`` configuration file.

    Sample Input::

        HTTPD=/usr/sbin/httpd.worker
        #
        # To pass additional options (for instance, -D definitions) to the
        # httpd binary at startup, set OPTIONS here.
        #
        OPTIONS=

    Examples:
        >>> httpd_syscfg['HTTPD']
        '/usr/sbin/httpd.worker'
        >>> httpd_syscfg.get('OPTIONS')
        ''
        >>> 'NOOP' in httpd_syscfg
        False
    """
    pass


@parser(Specs.sysconfig_irqbalance)
class IrqbalanceSysconfig(SysconfigOptions):
    """
    This parser analyzes the ``/etc/sysconfig/irqbalance`` configuration file.

    Sample Input::

        #IRQBALANCE_ONESHOT=yes
        #
        IRQBALANCE_BANNED_CPUS=f8
        IRQBALANCE_ARGS="-d"

    Examples:
        >>> irqb_syscfg['IRQBALANCE_BANNED_CPUS']
        'f8'
        >>> irqb_syscfg.get('IRQBALANCE_ARGS')  # quotes will be stripped
        '-d'
        >>> irqb_syscfg.get('IRQBALANCE_ONESHOT') is None
        True
        >>> 'ONESHOT' in irqb_syscfg
        False
    """
    pass


@parser(Specs.sysconfig_kdump)
class KdumpSysconfig(SysconfigOptions):
    """
    This parser reads data from the ``/etc/sysconfig/kdump`` file.

    This parser sets the following properties for ease of access:

    * KDUMP_COMMANDLINE
    * KDUMP_COMMANDLINE_REMOVE
    * KDUMP_COMMANDLINE_APPEND
    * KDUMP_KERNELVER
    * KDUMP_IMG
    * KDUMP_IMG_EXT
    * KEXEC_ARGS

    These are set to the value of the named variable in the kdump sysconfig
    file, or '' if not found.
    """
    KDUMP_KEYS = [
        'KDUMP_COMMANDLINE',
        'KDUMP_COMMANDLINE_REMOVE',
        'KDUMP_COMMANDLINE_APPEND',
        'KDUMP_KERNELVER',
        'KDUMP_IMG',
        'KDUMP_IMG_EXT',
        'KEXEC_ARGS',
    ]

    def parse_content(self, content):
        super(KdumpSysconfig, self).parse_content(content)
        for key in self.KDUMP_KEYS:
            setattr(self, key, self.data.get(key, ''))


@parser(Specs.sysconfig_kernel)
class KernelSysconfig(SysconfigOptions):
    """
    This parser reads data from the ``/etc/sysconfig/kernel`` file.

    Typical content example::

        # UPDATEDEFAULT specifies if new-kernel-pkg should make
        # new kernels the default
        UPDATEDEFAULT=yes

        # DEFAULTKERNEL specifies the default kernel package type
        DEFAULTKERNEL=kernel

    Examples:
        >>> kernel_syscfg.get('UPDATEDEFAULT')
        'yes'
        >>> 'DEFAULTKERNEL' in kernel_syscfg
        True
    """
    pass


@parser(Specs.sysconfig_libvirt_guests)
class LibvirtGuestsSysconfig(SysconfigOptions):
    """
    This parser analyzes the ``/etc/sysconfig/libvirt-guests`` configuration file.

    Sample Input::

        # URIs to check for running guests
        # example: URIS='default xen:/// vbox+tcp://host/system lxc:///'
        #URIS=default
        ON_BOOT=ignore

    Examples:
        >>> libvirt_guests_syscfg.get('ON_BOOT')
        'ignore'
    """
    pass


@parser(Specs.sysconfig_memcached)
class MemcachedSysconfig(SysconfigOptions):
    """
    This parser analyzes the ``/etc/sysconfig/memcached`` configuration file.

    Sample Input::

        PORT="11211"
        USER="memcached"
        # max connection 2048
        MAXCONN="2048"
        # set ram size to 2048 - 2GiB
        CACHESIZE="4096"
        # disable UDP and listen to loopback ip 127.0.0.1, for network connection use real ip e.g., 10.0.0.5
        OPTIONS="-U 0 -l 127.0.0.1"

    Examples:
        >>> memcached_syscfg.get('OPTIONS')
        '-U 0 -l 127.0.0.1'
    """
    pass


@parser(Specs.sysconfig_mongod)
class MongodSysconfig(SysconfigOptions):
    """
    A parser for analyzing the ``mongod`` service configuration file, like
    '/etc/sysconfig/mongod' and '/etc/opt/rh/rh-mongodb26/sysconfig/mongod'.

    Sample Input::

        OPTIONS="--quiet -f /etc/mongod.conf"

    Examples:
        >>> mongod_syscfg.get('OPTIONS')
        '--quiet -f /etc/mongod.conf'
        >>> mongod_syscfg.get('NO_SUCH_OPTION') is None
        True
        >>> 'NOSUCHOPTION' in mongod_syscfg
        False
    """
    pass


@parser(Specs.netconsole)
class NetconsoleSysconfig(SysconfigOptions):
    '''
    Parse the ``/etc/sysconfig/netconsole`` file.

    Sample Input::

        # The local port number that the netconsole module will use
        LOCALPORT=6666

    Examples:
        >>> 'LOCALPORT' in netcs_syscfg
        True
        >>> 'DEV' in netcs_syscfg
        False
    '''
    pass


@parser(Specs.sysconfig_network)
class NetworkSysconfig(SysconfigOptions):
    """
    This parser parses the ``/etc/sysconfig/network`` configuration file

    Sample Input::

        NETWORKING=yes
        HOSTNAME=rhel7-box
        GATEWAY=172.31.0.1
        NM_BOND_VLAN_ENABLED=no

    Examples:

        >>> 'NETWORKING' in net_syscfg
        True
        >>> net_syscfg['GATEWAY']
        '172.31.0.1'
    """
    pass


@parser(Specs.sysconfig_nfs)
class NfsSysconfig(SysconfigOptions):
    """
    A parser for analyzing the ``/etc/sysconfig/nfs`` configuration file.

    .. note::
        In some RHEL version, both the file ``/etc/nfs.conf`` and file
        ``/etc/sysconfig/nfs`` exist, and take effect at the same time for NFS
        services on the host. And it's possible that there are overlaps between
        the two configuration files.
        A combiner for the two configuration files was considered.
        Since the two files' coverage are different, and it is quite complicate
        to enumerate all the configuration options and combine them properly,
        and also, ``/etc/sysconfig/nfs`` is deprecated in lately RHEL releases.
        We deselect it as a consequence.

    Sample Input::

      RPCNFSDARGS="--rdma=20049"
      #STATD_PORT=662

    Examples:
        >>> 'RPCNFSDARGS' in nfs_syscfg
        True
        >>> nfs_syscfg['RPCNFSDARGS']
        '--rdma=20049'
        >>> 'STATD_PORT' in nfs_syscfg
        False
    """
    pass


@parser(Specs.sysconfig_ntpd)
class NtpdSysconfig(SysconfigOptions):
    """
    A parser for analyzing the ``/etc/sysconfig/ntpd`` configuration file.

    Sample Input::

      OPTIONS="-x -g"
      #HIDE="me"

    Examples:
        >>> 'OPTIONS' in ntpd_syscfg
        True
        >>> 'HIDE' in ntpd_syscfg
        False
        >>> ntpd_syscfg['OPTIONS']
        '-x -g'
    """
    pass


@parser(Specs.sysconfig_prelink)
class PrelinkSysconfig(SysconfigOptions):
    """
    A parser for analyzing the ``/etc/sysconfig/prelink`` configuration file.

    Sample Input::

        # Set this to no to disable prelinking altogether
        # (if you change this from yes to no prelink -ua
        # will be run next night to undo prelinking)
        PRELINKING=no

        # Options to pass to prelink
        # -m    Try to conserve virtual memory by allowing overlapping
        #       assigned virtual memory slots for libraries which
        #       never appear together in one binary
        # -R    Randomize virtual memory slot assignments for libraries.
        #       This makes it slightly harder for various buffer overflow
        #       attacks, since library addresses will be different on each
        #       host using -R.
        PRELINK_OPTS=-mR

    Examples:
        >>> prelink_syscfg.get('PRELINKING')
        'no'
    """
    pass


@parser(Specs.sysconfig_sbd)
class SbdSysconfig(SysconfigOptions):
    """
    A parser for analyzing the ``/etc/sysconfig/sbd`` configuration file.

    Sample Input::

        SBD_DEVICE="/dev/disk/by-path/ip-198.20.25.163:3260-iscsi-iqn.2022-09.com.epmpttest:rhel85-lun-0"
        SBD_DELAY_START=no
        SBD_OPTS="-n emptytest"
        SBD_PACEMAKER=yes
        SBD_STARTMODE=always
        SBD_WATCHDOG_DEV=/dev/watchdog
        SBD_WATCHDOG_TIMEOUT=5

    Examples:
        >>> sbd_syscfg.get('SBD_WATCHDOG_TIMEOUT')
        '5'
        >>> 'SBD_DELAY_START' in sbd_syscfg
        True
    """
    pass


@parser(Specs.sysconfig_sshd)
class SshdSysconfig(SysconfigOptions):
    """
    A parser for analyzing the ``/etc/sysconfig/sshd`` configuration file.

    Sample Input::

        # Configuration file for the sshd service.

        # The server keys are automatically generated if they are missing.
        # To change the automatic creation, adjust sshd.service options for
        # example using  systemctl enable sshd-keygen@dsa.service  to allow creation
        # of DSA key or  systemctl mask sshd-keygen@rsa.service  to disable RSA key
        # creation.

        # System-wide crypto policy:
        # To opt-out, uncomment the following line
        # CRYPTO_POLICY=
        CRYPTO_POLICY=

    Examples:
        >>> sshd_syscfg.get('CRYPTO_POLICY')
        ''
        >>> 'NONEXISTENT_VAR' in sshd_syscfg
        False
        >>> 'CRYPTO_POLICY' in sshd_syscfg
        True
    """
    pass


@parser(Specs.puppetserver_config)
class PuppetserverSysconfig(SysconfigOptions):
    """
    Parse the ``/etc/sysconfig/puppetserver`` configuration file.

    Sample configuration file::

        USER="puppet"
        GROUP="puppet"
        INSTALL_DIR="/opt/puppetlabs/server/apps/puppetserver"
        CONFIG="/etc/puppetlabs/puppetserver/conf.d"
        START_TIMEOUT=300

    Examples:
        >>> pps_syscfg['START_TIMEOUT']
        '300'
        >>> 'AUTO' in pps_syscfg
        False
    """
    pass


@parser(Specs.sysconfig_stonith)
class StonithSysconfig(SysconfigOptions):
    """
    Class to parse the ``/etc/sysconfig/stonith``

    Sample Input::

        retry=3
        retry-sleep=2
        verbose=yes    # optional

    Examples:
        >>> 'retry' in stonith_syscfg
        True
        >>> stonith_syscfg['retry']
        '3'
    """
    pass


@parser(Specs.up2date)
class Up2DateSysconfig(SysconfigOptions):
    """
    Class to parse the ``/etc/sysconfig/rhn/up2date``

    Typical content example::

        serverURL[comment]=Remote server URL
        #serverURL=https://rhnproxy.glb.tech.markit.partners
        serverURL=https://rhnproxy.glb.tech.markit.partners/XMLRPC

    Examples:
        >>> 'serverURL' in u2d_syscfg
        True
        >>> u2d_syscfg['serverURL']
        'https://rhnproxy.glb.tech.markit.partners/XMLRPC'
    """
    def parse_content(self, content):
        up2date_info = {}
        for line in get_active_lines(content):
            if "[comment]" not in line and '=' in line:
                key, val = line.split('=')
                up2date_info[key.strip()] = val.strip()
        self.data = up2date_info


@parser(Specs.sysconfig_virt_who)
class VirtWhoSysconfig(SysconfigOptions):
    """
    A parser for analyzing the ``/etc/sysconfig/virt-who`` configuration file.

    Sample Input::

        # Register ESX machines using vCenter
        # VIRTWHO_ESX=0
        # Register guests using RHEV-M
        VIRTWHO_RHEVM=1

        # Options for RHEV-M mode
        VIRTWHO_RHEVM_OWNER=
        TEST_OPT="A TEST"

    Examples:
        >>> vwho_syscfg['VIRTWHO_RHEVM']
        '1'
        >>> vwho_syscfg.get('VIRTWHO_RHEVM_OWNER')
        ''
        >>> vwho_syscfg.get('NO_SUCH_OPTION') is None
        True
        >>> 'NOSUCHOPTION' in vwho_syscfg
        False
        >>> vwho_syscfg.get('TEST_OPT')  # Quotes are stripped
        'A TEST'
    """
    pass


@parser(Specs.ifcfg_static_route)
class IfCFGStaticRoute(SysconfigOptions):
    """
    IfCFGStaticRoute is a parser for the static route network interface
    definition files in ``/etc/sysconfig/network-scripts``.  These are
    pulled into the network scripts using ``source``, so they are mainly
    ``bash`` environment declarations of the form **KEY=value**.  These
    are stored in the ``data`` property as a dictionary.  Quotes surrounding
    the value

    Because this parser reads multiple files, the interfaces are stored as a
    list within the parser and need to be iterated through in order to find
    specific interfaces.

    Sample configuration from a static connection in file ``/etc/sysconfig/network-scripts/rute-test-net``::

        ADDRESS0=10.65.223.0
        NETMASK0=255.255.254.0
        GATEWAY0=10.65.223.1

    Examples:

        >>> conn_info['ADDRESS0']
        '10.65.223.0'
        >>> conn_info.static_route_name
        'test-net'

    Attributes:
        static_route_name (str): static route name

    """
    def parse_content(self, content):
        self.static_route_name = self.file_name.split("route-", 1)[1]
        super(IfCFGStaticRoute, self).parse_content(content)


@parser(Specs.sysconfig_grub)
class GrubSysconfig(SysconfigOptions):
    """
    Class to parse the ``/etc/sysconfig/grub``

    ``/etc/sysconfig/grub`` is a symlink of ``/etc/default/grub`` file

    Typical content example::

        GRUB_TIMEOUT=1
        GRUB_DISTRIBUTOR="$(sed 's, release .*$,,g' /etc/system-release)"
        GRUB_DEFAULT=saved
        GRUB_DISABLE_SUBMENU=true
        GRUB_TERMINAL_OUTPUT="console"
        GRUB_CMDLINE_LINUX="console=ttyS0 console=ttyS0,115200n8 no_timer_check net.ifnames=0 crashkernel=auto"
        GRUB_DISABLE_RECOVERY="true"
        GRUB_ENABLE_BLSCFG=true

    Examples:
        >>> grub_syscfg.get('GRUB_ENABLE_BLSCFG')
        'true'
        >>> 'NONEXISTENT_VAR' in grub_syscfg
        False
        >>> 'GRUB_ENABLE_BLSCFG' in grub_syscfg
        True

    """
    pass


@parser(Specs.sysconfig_oracleasm)
class OracleasmSysconfig(SysconfigOptions):
    """
    Class to parse the ``/etc/sysconfig/oracleasm``

    Typical content example::

        #
        # This is a configuration file for automatic loading of the Oracle
        # Automatic Storage Management library kernel driver.  It is generated
        # By running /etc/init.d/oracleasm configure.  Please use that method
        # to modify this file
        #

        # ORACLEASM_ENABELED: 'true' means to load the driver on boot.
        ORACLEASM_ENABLED=true

        # ORACLEASM_UID: Default user owning the /dev/oracleasm mount point.
        ORACLEASM_UID=oracle

        # ORACLEASM_GID: Default group owning the /dev/oracleasm mount point.
        ORACLEASM_GID=oinstall

        # ORACLEASM_SCANBOOT: 'true' means scan for ASM disks on boot.
        ORACLEASM_SCANBOOT=true

        # ORACLEASM_SCANORDER: Matching patterns to order disk scanning
        ORACLEASM_SCANORDER="dm"

        # ORACLEASM_SCANEXCLUDE: Matching patterns to exclude disks from scan
        ORACLEASM_SCANEXCLUDE="sd"

    Examples:
        >>> oracleasm_syscfg.get('ORACLEASM_SCANBOOT')
        'true'
        >>> 'ORACLEASM_SCANORDER' in oracleasm_syscfg
        True
        >>> 'ORACLEASM_SCANEXCLUDE_1' in oracleasm_syscfg
        False

    """
    pass


@parser(Specs.sysconfig_pcsd)
class PcsdSysconfig(SysconfigOptions):
    """
    Class to parse the ``/etc/sysconfig/pcsd``

    Typical content example::

        # Set PCSD_DEBUG to true for advanced pcsd debugging information
        PCSD_DEBUG=false
        # Set DISABLE_GUI to true to disable GUI frontend in pcsd
        PCSD_DISABLE_GUI=false
        # Set web UI sesions lifetime in seconds
        PCSD_SESSION_LIFETIME=3600

    Examples:
        >>> pcsd_syscfg.get('PCSD_DEBUG')
        'false'
        >>> 'PCSD_DISABLE_GUI' in pcsd_syscfg
        True
        >>> pcsd_syscfg.get('PCSD_SESSION_LIFETIME')
        '3600'

    """
    pass



import doctest
from insights.parsers import neutron_l3_agent_conf
from insights.parsers.neutron_l3_agent_conf import NeutronL3AgentIni
from insights.tests import context_wrap


L3_AGENT_INI = """
[DEFAULT]

#
# From neutron.base.agent
#

# Name of Open vSwitch bridge to use (string value)
ovs_integration_bridge = br-int

# Uses veth for an OVS interface or not. Support kernels with limited namespace
# support (e.g. RHEL 6.5) so long as ovs_use_veth is set to True. (boolean
# value)
ovs_use_veth = false

# The driver used to manage the virtual interface. (string value)
#interface_driver = <None>
interface_driver = neutron.agent.linux.interface.OVSInterfaceDriver

# Timeout in seconds for ovs-vsctl commands. If the timeout expires, ovs
# commands will fail with ALARMCLOCK error. (integer value)
ovs_vsctl_timeout = 10

#
# From neutron.l3.agent
#

# The working mode for the agent. Allowed modes are: 'legacy' - this preserves
# the existing behavior where the L3 agent is deployed on a centralized
# networking node to provide L3 services like DNAT, and SNAT. Use this mode if
# you do not want to adopt DVR. 'dvr' - this mode enables DVR functionality and
# must be used for an L3 agent that runs on a compute host. 'dvr_snat' - this
# enables centralized SNAT support in conjunction with DVR.  This mode must be
# used for an L3 agent running on a centralized node (or in single-host
# deployments, e.g. devstack) (string value)
# Allowed values: dvr, dvr_snat, legacy
agent_mode = dvr

# TCP Port used by Neutron metadata namespace proxy. (port value)
# Minimum value: 0
# Maximum value: 65535
metadata_port = 9697

# Send this many gratuitous ARPs for HA setup, if less than or equal to 0, the
# feature is disabled (integer value)
#send_arp_for_ha = 3

# Allow running metadata proxy. (boolean value)
enable_metadata_proxy = true

# DEPRECATED: Name of bridge used for external network traffic. When this
# parameter is set, the L3 agent will plug an interface directly into an
# external bridge which will not allow any wiring by the L2 agent. Using this
# will result in incorrect port statuses. This option is deprecated and will be
# removed in Ocata. (string value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
external_network_bridge =

#
# From oslo.log
#

# If set to true, the logging level will be set to DEBUG instead of the default
# INFO level. (boolean value)
# Note: This option can be changed without restarting.
debug = False

# Defines the format string for %%(asctime)s in log records. Default:
# %(default)s . This option is ignored if log_config_append is set. (string
# value)
log_date_format = %Y-%m-%d %H:%M:%S

[AGENT]

#
# From neutron.base.agent
#

# Seconds between nodes reporting state to server; should be less than
# agent_down_time, best if it is half or less than agent_down_time. (floating
# point value)
report_interval = 30

# Log agent heartbeats (boolean value)
log_agent_heartbeats = false

# Availability zone of this node (string value)
availability_zone = nova
""".strip()


def test_neutron_l3_agent_ini():
    nla_ini = NeutronL3AgentIni(context_wrap(L3_AGENT_INI))
    assert nla_ini.has_option("AGENT", "log_agent_heartbeats")
    assert nla_ini.get("DEFAULT", "agent_mode") == "dvr"
    assert nla_ini.getint("DEFAULT", "metadata_port") == 9697


def test_doc():
    env = {"l3_agent_ini": NeutronL3AgentIni(context_wrap(L3_AGENT_INI, path="/etc/neutron/l3_agent.ini"))}
    failed, total = doctest.testmod(neutron_l3_agent_conf, globs=env)
    assert failed == 0



"""
Subscription manager list outputs - command ``subscription-manager list``
=========================================================================

This module provides parsers for various list outputs of ``subscription-manager``.

Parsers provided by this module are:

SubscriptionManagerListConsumed - command ``subscription-manager list --consumed``
----------------------------------------------------------------------------------

SubscriptionManagerListInstalled - command ``subscription-manager list --installed``
------------------------------------------------------------------------------------
"""
import re
from datetime import datetime
import six
from insights.specs import Specs
from .. import parser, CommandParser
from . import keyword_search


class SubscriptionManagerList(CommandParser):
    """
    A general object for parsing the output of ``subscription-manager list``.
    This should be subclassed to read the specific output - e.g. ``--consumed``
    or ``--installed``.

    Attributes:
        records (list): A list of dict with the output info, it's empty when the ``error`` occurs
        error (str): The raised exception when there is traceback
    """
    def parse_content(self, content):
        self.records = []
        current_record = {}
        record_start_key = ''
        key = ''  # The key currently in use

        key_val_re = re.compile(r'^(?P<key>\w[\w\s]+\w):\s+(?P<value>\w.*)$')
        cont_val_re = re.compile(r'^\s+(?P<value>\w.*)$')

        # The first line that matches the key_val_re is treated as the start
        # of a record.  The header doesn't match because it doesn't have a
        # colon.  Once that's read, if the record start key is seen again
        # it starts a new record.

        for line in content:
            if 'Traceback' in line:
                self.error = content[-1]
                break
            # Check for match of key/value line
            match = key_val_re.search(line)
            if match:
                key, value = match.group('key', 'value')
                if not record_start_key:
                    record_start_key = key
                # Have we started a new record
                if key == record_start_key:
                    # If we have an existing record, save it
                    if current_record:
                        self.records.append(current_record)
                    current_record = {}
                current_record[key] = value
                # Do some type conversions and add-ons
                if key == 'Active' and value in ('True', 'False'):
                    current_record[key] = (value == 'True')
                elif key in ('Starts', 'Ends'):
                    try:
                        current_record[key + ' timestamp'] = datetime.strptime(
                            value, '%m/%d/%y'
                        )
                    except ValueError:
                        pass
            elif not record_start_key:
                # Ignore any lines before the first record key.
                continue
            # Check for value continuations
            match = cont_val_re.search(line)
            if match:
                # Add this value to the current key:
                if isinstance(current_record[key], six.string_types):
                    # Convert the single string into a list
                    current_record[key] = [
                        current_record[key], match.group('value')
                    ]
                else:
                    current_record[key].append(match.group('value'))

        # Save the last read record if we had one.
        if current_record:
            self.records.append(current_record)

    def search(self, *args, **kwargs):
        """
        Search for records that match the given keys and values.  See the
        :func:`insights.parsers.keyword_search` function for more details
        on usage.

        Arguments:
            **kwargs: Key-value pairs of search parameters.

        Returns:
            (list): A list of records that matched the search criteria.

        Examples:
            >>> len(consumed.search(Service_Level='PREMIUM'))
            1
            >>> consumed.search(Provides__contains='Red Hat Enterprise Virtualization')
            []
        """
        return keyword_search(self.records, **kwargs)


@parser(Specs.subscription_manager_list_consumed)
class SubscriptionManagerListConsumed(SubscriptionManagerList):
    """
    Read the output of ``subscription-manager list --consumed``.

    Sample input file::

        +-------------------------------------------+
           Consumed Subscriptions
        +-------------------------------------------+
        Subscription Name: Red Hat Enterprise Linux Server, Premium (1-2 sockets) (Up to 1 guest)
        Provides:          Oracle Java (for RHEL Server)
                           Red Hat Software Collections Beta (for RHEL Server)
                           Red Hat Enterprise Linux Server
                           Red Hat Beta
        SKU:               RH0155783S
        Contract:          12345678
        Account:           1000001
        Serial:            0102030405060708090
        Pool ID:           8a85f981477e5284014783abaf5d4dcd
        Active:            True
        Quantity Used:     1
        Service Level:     PREMIUM
        Service Type:      L1-L3
        Status Details:    Subscription is current
        Subscription Type: Standard
        Starts:            11/14/14
        Ends:              07/06/15
        System Type:       Physical

    Examples:
        >>> type(consumed)
        <class 'insights.parsers.subscription_manager_list.SubscriptionManagerListConsumed'>
        >>> len(consumed.records)
        1
        >>> sub1 = consumed.records[0]
        >>> sub1['SKU']
        'RH0155783S'
        >>> sub1['Active']  # Type conversion on Active field
        True
        >>> sub1['Status Details']  # Keys appear as given
        'Subscription is current'
        >>> sub1['Provides'][1]
        'Red Hat Software Collections Beta (for RHEL Server)'
        >>> sub1['Starts']  # Basic field as text - note month/day/year
        '11/14/14'
        >>> sub1['Starts timestamp'].year
        2014
        >>> consumed.all_current  # Are all subscriptions listed as current?
        True
    """
    @property
    def all_current(self):
        """
        (bool) Does every subscription record have the Status Details value
        set to 'Subscription is current'?
        """
        return all(
            sub['Status Details'] == 'Subscription is current'
            for sub in self.records
        )


@parser(Specs.subscription_manager_list_installed)
class SubscriptionManagerListInstalled(SubscriptionManagerList):
    """
    Read the output of ``subscription-manager list --installed``.

    Sample input file::

        +-------------------------------------------+
        Installed Product Status
        +-------------------------------------------+
        Product Name:   Red Hat Software Collections (for RHEL Server)
        Product ID:     201
        Version:        2
        Arch:           x86_64
        Status:         Subscribed
        Status Details:
        Starts:         04/27/15
        Ends:           04/27/16

        Product Name:   Red Hat Enterprise Linux Server
        Product ID:     69
        Version:        7.1
        Arch:           x86_64
        Status:         Subscribed
        Status Details:
        Starts:         04/27/15
        Ends:           04/27/16

    Examples:
        >>> type(installed)
        <class 'insights.parsers.subscription_manager_list.SubscriptionManagerListInstalled'>
        >>> len(installed.records)
        2
        >>> prod1 = installed.records[0]
        >>> prod1['Product ID']  # Note - not converted to number
        '201'
        >>> prod1['Starts']  # String date as is
        '04/27/15'
        >>> prod1['Starts timestamp'].year  # Extra converted to date
        2015
        >>> installed.all_subscribed
        True

    """
    @property
    def all_subscribed(self):
        """
        (bool) Does every product record have the Status value set to
        'Subscribed'?
        """
        return all(
            sub['Status'] == 'Subscribed'
            for sub in self.records
        )



"""
Custom datasources related to ``httpd``
"""
import glob
import json
import os

from insights.combiners.ps import Ps
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.parsers.mount import ProcMounts
from insights.specs.datasources import get_running_commands


@datasource(Ps, HostContext)
def httpd_cmds(broker):
    """
    Function to search the output of ``ps auxcww`` to find all running Apache
    webserver processes and extract the binary path.

    Returns:
        list: List of the binary paths to each running process
    """
    cmds = get_running_commands(broker[Ps], broker[HostContext], ['httpd', ])
    if cmds:
        return cmds
    raise SkipComponent


@datasource(ProcMounts, HostContext)
def httpd_on_nfs(broker):
    """
    Function to get the count of httpd opened file on nfs v4

    Returns:
        str: JSON string with keys: "httpd_ids", "nfs_mounts", "open_nfs_files"
    """
    mnt = broker[ProcMounts]
    mps = mnt.search(mount_type='nfs4')
    # get nfs 4.0 mount points
    nfs_mounts = [m.mount_point for m in mps if 'vers' in m.mount_options and m.mount_options.vers.startswith("4")]
    if nfs_mounts:
        # get all httpd ps
        httpd_pids = broker[HostContext].shell_out("pgrep httpd")
        if httpd_pids:
            open_nfs_files = 0
            lsof_cmds = ["lsof -p {0}".format(pid) for pid in httpd_pids if pid]
            # maybe there are thousands open files
            for cmd in lsof_cmds:
                for line in broker[HostContext].shell_out(cmd):
                    items = line.split()
                    if len(items) > 8 and items[8].startswith(tuple(nfs_mounts)):
                        open_nfs_files += 1
            result_dict = {"http_ids": httpd_pids, "nfs_mounts": nfs_mounts, "open_nfs_files": open_nfs_files}
            relative_path = 'insights_commands/httpd_open_nfsV4_files'
            return DatasourceProvider(content=json.dumps(result_dict), relative_path=relative_path)
    raise SkipComponent


def _get_all_include_conf(root, glob_path):
    includes = glob_path
    # In case $ServerRoot in included in the 'glob_path'
    if not glob_path.startswith(root):
        includes = os.path.join(root, glob_path)
    _paths = set()
    try:
        for conf in glob.glob(includes):
            if os.path.isfile(conf):
                _paths.add(conf)
                with open(conf) as cfp:
                    _includes = None
                    section_number = 0
                    for line in cfp.readlines():
                        line = line.strip()
                        if line.startswith("</"):
                            section_number = section_number - 1
                        elif line.startswith("<"):
                            section_number = section_number + 1
                        if section_number == 0 and (line.startswith("Include ") or line.startswith("IncludeOptional ")):
                            _includes = line.split()[-1].strip('"\'')
                            _paths.update(_get_all_include_conf(root, _includes))
            if os.path.isdir(conf):
                _includes = os.path.join(conf, "*")
                _paths.update(_get_all_include_conf(root, _includes))
        return _paths
    except Exception:
        pass
    return _paths


def get_httpd_configuration_files(httpd_root):
    main_httpd_conf = os.path.join(httpd_root, "conf/httpd.conf")
    all_paths = set()
    try:
        with open(main_httpd_conf) as cfp:
            server_root = httpd_root
            # Add it only when it exists
            all_paths.add(main_httpd_conf)
            section_number = 0
            for line in cfp.readlines():
                line = line.strip()
                if line.startswith("</"):
                    section_number = section_number - 1
                elif line.startswith("<"):
                    section_number = section_number + 1
                if line.startswith("ServerRoot "):
                    server_root = line.strip().split()[-1].strip().strip('"\'')
                elif section_number == 0 and (line.startswith("Include ") or line.startswith("IncludeOptional ")):
                    includes = line.split()[-1].strip('"\'')
                    # For multiple "Include" directives, all of them will be included
                    all_paths.update(_get_all_include_conf(server_root, includes))
    except Exception:
        # Skip the datasource when no such "<root path>/httpd.conf" file
        raise SkipComponent
    return all_paths


@datasource(HostContext)
def httpd_configuration_files(broker):
    """
    This datasource returns the all of httpd configuration files' path.

    Returns:
        list: the file path of httpd configuration files

    Raises:
        SkipComponent: there is no httpd configuration file
    """
    httpd_root = '/etc/httpd'
    return get_httpd_configuration_files(httpd_root)


@datasource(HostContext)
def httpd24_scl_configuration_files(broker):
    """
    This datasource returns the all of httpd24 slc configuration files' path.

    Returns:
        list: the file path of httpd24 slc configuration files

    Raises:
        SkipComponent: there is no httpd24 slc configuration file
    """
    httpd_root = '/opt/rh/httpd24/root/etc/httpd'
    return get_httpd_configuration_files(httpd_root)


@datasource(HostContext)
def httpd24_scl_jbcs_configuration_files(broker):
    """
    This datasource returns the all of httpd24 slc jbcs configuration files' path.

    Returns:
        list: the file path of httpd24 slc jbcs configuration files

    Raises:
        SkipComponent: there is no httpd24 slc jbcs configuration file
    """
    httpd_root = '/opt/rh/jbcs-httpd24/root/etc/httpd'
    return get_httpd_configuration_files(httpd_root)



"""
DMIDecode - Command ``dmidecode``
=================================

Parses the output of the ``dmidecode`` command to catalogue the hardware
associated with the system.

In general, DMIdecode recognizes the sections of device information,
separated by blank lines, processed in the following way.

* It uses the descriptor line that precedes the indented device information
  (e.g. 'BIOS Information') as the name for that section, converting the name
  into lower case and replacing spaces with underscores (e.g.
  'bios_information') to look more Pythonic.

* Within each section, data is split up on colons.

* Lines such as 'Characteristics' that end with a colon and have one or more
  further indented lines after them are converted into a list of the values so
  indented.

The common information retrieved from dmidecode is available in several
convenience properties:

* **system_info** - Information about the machine itself
* **bios** - the BIOS information
* **bios_vendor** - the BIOS's 'Vendor' attribute
* **bios_date** - the BIOS's 'Release Date' attribute
* **processor_manufacturer** - the processor's 'Manufacturer' attribute
* **is_present** - this indicates whether dmidecode information was found.

Sample input::

    # dmidecode 2.2
    SMBIOS 2.4 present.
    104 structures occupying 3162 bytes.
    Table at 0x000EE000.
    Handle 0x0000
        DMI type 0, 24 bytes.
        BIOS Information
            Vendor: HP
            Version: A08
            Release Date: 09/27/2008
            Address: 0xF0000
            Runtime Size: 64 kB
            ROM Size: 4096 kB
            Characteristics:
                PCI is supported
                PNP is supported
                BIOS is upgradeable
                BIOS shadowing is allowed
                ESCD support is available
                Boot from CD is supported
                Selectable boot is supported
                EDD is supported
                5.25"/360 KB floppy services are supported (int 13h)
                5.25"/1.2 MB floppy services are supported (int 13h)
                3.5"/720 KB floppy services are supported (int 13h)
                Print screen service is supported (int 5h)
                8042 keyboard services are supported (int 9h)
                Serial services are supported (int 14h)
                Printer services are supported (int 17h)
                CGA/mono video services are supported (int 10h)
                ACPI is supported
                USB legacy is supported
                BIOS boot specification is supported
                Function key-initiated network boot is supported``
    Handle 0x0100
        DMI type 1, 27 bytes.
        System Information
            Manufacturer: HP
            Product Name: ProLiant BL685c G1
            Version: Not Specified
            Serial Number: 3H6CMK2537
            UUID: 58585858-5858-3348-3643-4D4B32353337
            Wake-up Type: Power Switch

Examples:

    >>> dmi = shared[DMIDecode]
    >>> dmi.is_present
    True
    >>> len(dmi['bios_information'])
    1
    >>> dmi['bios_information'][0]['vendor']
    'HP'
    >>> dmi.bios_vendor
    'HP'
    >>> dmi.bios_date
    datetime.date(2008, 9, 27)
    >>> len(dmi.bios['characteristics'])
    20
    >>> dmi.bios['characteristics'][0]
    'PCI is supported'
    >>> dmi.system_uuid == '58585858-5858-3348-3643-4D4B32353337'
    True
"""
import re
import uuid

from datetime import date

from insights import LegacyItemAccess, parser, defaults, CommandParser
from insights.specs import Specs


@parser(Specs.dmidecode)
class DMIDecode(CommandParser, LegacyItemAccess):
    """
    Class for DMI information.
    """

    def parse_content(self, content):
        self.data = parse_dmidecode(content, pythonic_keys=True)

    @property
    def system_info(self):
        """(str): Convenience method to get system information"""
        return self["system_information"][0] if "system_information" in self else None

    @property
    @defaults()
    def system_uuid(self):
        """(str): Convenience method to get system UUID"""
        sys_uuid = self["system_information"][0]['uuid']
        return str(uuid.UUID(sys_uuid))

    @property
    def bios(self):
        """(str): Convenience method to get BIOS information"""
        return self["bios_information"][0] if "bios_information" in self else None

    @property
    @defaults()
    def bios_vendor(self):
        """(str): Convenience method to get BIOS vendor"""
        return self["bios_information"][0]["vendor"]

    @property
    @defaults()
    def bios_date(self):
        """(datetime.date): Get the BIOS release date in datetime.date format"""
        month, day, year = map(int, self["bios_information"][0]["release_date"].split("/"))
        return date(year, month, day)

    @property
    @defaults()
    def processor_manufacturer(self):
        """(str): Convenience method to get the processor manufacturer"""
        return self["processor_information"][0]["manufacturer"]

    @property
    def is_present(self):
        """(bool): Is there any decodable data in the content?"""
        return bool(self.data)


def parse_dmidecode(dmidecode_content, pythonic_keys=False):
    """
    Returns a dictionary of dmidecode information parsed from a dmidecode list
    (i.e. from context.content)

    This method will attempt to handle leading spaces rather than tabs.
    """
    if len(dmidecode_content) < 3:
        return {}

    section = None
    obj = {}
    current = {}

    buf = "\n".join(dmidecode_content).strip()

    # Some versions of DMIDecode have an extra
    # level of indentation, as well as slightly
    # different configuration for each section.
    if "\tDMI type" in buf:
        pat = re.compile("^\t", flags=re.MULTILINE)
        buf = pat.sub("", buf)

        buf = buf.replace("\nDMI type", "DMI type")
        buf = buf.replace("\nHandle", "\n\nHandle")

    buf = buf.replace("\n\t\t", "\t")

    def fix_key(k):
        return k.lower().replace(" ", "_") if pythonic_keys else k

    for line in buf.splitlines():
        nbline = line.strip()
        if section:
            if not nbline:
                # There maybe some sections with the same name, such as:
                # processor_information
                if section in obj:
                    obj[section].append(current)
                else:
                    obj[section] = [current]
                current = {}
                section = key = None
                continue

            elif line.startswith("\t"):
                if ":" in line:
                    key, value = nbline.split(":", 1)
                    key = fix_key(key)
                    value = value.strip()

                if "\t" in value:
                    current[key] = list(filter(None, value.split("\t")))
                else:
                    current[key] = value
            else:
                section = None

        if not section:
            # Ignore 'Table at 0xBFFCB000' and similar.
            if not ('Table' in nbline or 'table' in nbline):
                section = fix_key(nbline)

    if section in obj:
        obj[section].append(current)
    else:
        obj[section] = [current]

    return obj



# -*- coding: utf-8 -*-
import pytest
import sys
import warnings

from insights.tests import deep_compare
from insights.core.dr import split_requirements, stringify_requirements, get_missing_requirements
from insights.core import context
from insights.util import case_variants, deprecated


class t(object):

    def __init__(self, n):
        self.serializable_id = self.n = n

    def __eq__(self, item):
        return hasattr(item, "n") and item.n == self.n

    def __lt__(self, item):
        return self.n < item.n

    def __hash__(self):
        return self.n.__hash__()

    def __repr__(self):
        return self.n


def dummy_dict(l):
    return dict((k, 1) for k in l)


REQUIRES = [t("a"), t("b"), [t("d"), t("e")], t("c"), [t("f"), t("g")]]


def test_split():
    expected = ([t("a"), t("b"), t("c")], [[t("d"), t("e")], [t("f"), t("g")]])
    result = split_requirements(REQUIRES)
    assert expected == result


def test_stringify():
    expected = "All: ['a', 'b', 'c'] Any: ['d', 'e'] Any: ['f', 'g']"
    result = stringify_requirements(REQUIRES)
    assert expected == result


def test_missing():
    missing_none = dummy_dict([t("a"), t("b"), t("c"), t("d"), t("e"), t("f"), t("g")])
    assert get_missing_requirements(None, REQUIRES, missing_none) is None
    missing_partial_any = dummy_dict([t("a"), t("b"), t("c"), t("d"), t("g")])
    assert get_missing_requirements(None, REQUIRES, missing_partial_any) is None
    missing_any = dummy_dict([t("a"), t("b"), t("c"), t("f"), t("g")])
    result = get_missing_requirements(None, REQUIRES, missing_any)
    assert result == ([], [[t("d"), t("e")]])
    missing_all = dummy_dict([t("a"), t("c"), t("d"), t("e"), t("f"), t("g")])
    result = get_missing_requirements(None, REQUIRES, missing_all)
    assert result == ([t("b")], [])


@pytest.fixture
def ctx():
    return context.Context()


def test_sane_defaults(ctx):
    assert ctx.hostname is None
    assert ctx.version == ["-1", "-1"]


def test_non_product_member(ctx):
    with pytest.raises(AttributeError):
        ctx.invalid_product()


def test_str():
    deep_compare("foo", "foo")
    deep_compare(u"foo", u"foo")
    with pytest.raises(AssertionError):
        deep_compare("foo", "bar")


def test_num():
    deep_compare(1, 1)
    deep_compare(1.1, 1.1)
    with pytest.raises(AssertionError):
        deep_compare(1, 2)


def test_list():
    deep_compare([1, 2, 3], [1, 2, 3])
    with pytest.raises(AssertionError):
        deep_compare([2, 3, 4], [4, 3, 2])
        deep_compare([1, 2, 3], [2, 3, 4])


def test_dict():
    deep_compare({"foo": 1}, {"foo": 1})
    deep_compare({"foo": 1, "bar": 2}, {"bar": 2, "foo": 1})
    with pytest.raises(AssertionError):
        deep_compare({"foo": 1}, {"foo": 2})

    with pytest.raises(AssertionError):
        deep_compare({"foo": 1, "bar": [1, 2, 3]}, {"foo": 1, "bar": [0, 1, 2]})


def test_beautify_deep_compare_diff():
    with pytest.raises(AssertionError) as err:
        deep_compare({"foo": "some foo"}, {"bar": "some bar"})
    einfo = None
    if hasattr(err, "value"):       # py3
        einfo = err.value
    elif hasattr(err, "message"):   # py2
        einfo = err.message
    assert 'key "foo" not in Expected;' in str(einfo)
    assert 'key "bar" not in Result;' in str(einfo)

    with pytest.raises(AssertionError) as err:
        deep_compare({"e": "k", "common": "left"}, {"e": "k", "common": "right"})
    einfo = None
    if hasattr(err, "value"):       # py3
        einfo = err.value
    elif hasattr(err, "message"):   # py2
        einfo = err.message
    assert 'key "common" unequal values:' in str(einfo)
    assert 'Result: ' in str(einfo)


def test_deep_nest():
    a = {"error_key": "test1", "stuff": {"abba": [{"foo": 2}]}}
    b = {"error_key": "test1", "stuff": {"abba": [{"foo": 2}]}}

    deep_compare(a, b)

    with pytest.raises(AssertionError):
        b["stuff"]["abba"][0]["foo"] = "cake"
        deep_compare(a, b)


def test_deep_nest_tuple_dict():
    a = ({2: 22}, {3: 33}, {4: 44})
    b = ({2: 22}, {3: 33}, {4: 44})
    deep_compare(a, b)
    with pytest.raises(AssertionError):
        b = ({2: 22}, {3: 30}, {4: 44})
        deep_compare(a, b)

    a = ({2: 22}, {3: [33, 333]}, {4: 44})
    b = ({2: 22}, {3: [33, 333]}, {4: 44})
    deep_compare(a, b)
    with pytest.raises(AssertionError):
        b = ({2: 22}, {3: [333, 33]}, {4: 44})
        deep_compare(a, b)

    a = ({2: 22, 5: '55'}, [33, 333, '3'], {6: [6], 4: 44})
    b = ({5: '55', 2: 22}, [33, 333, '3'], {4: 44, 6: [6]})
    deep_compare(a, b)
    with pytest.raises(AssertionError):
        b = ({5: '55', 2: 22}, [33, 333, 3], {4: 44, 6: [6]})
        deep_compare(a, b)


def test_deep_nest_list_dict():
    a = [{2: 22}, {3: 33}, {4: 44}]
    b = [{2: 22}, {3: 33}, {4: 44}]
    deep_compare(a, b)
    with pytest.raises(AssertionError):
        b[1][3] = 30
        deep_compare(a, b)

    a = [{2: 22}, {3: [33, 333]}, {4: 44}]
    b = [{2: 22}, {3: [33, 333]}, {4: 44}]
    deep_compare(a, b)
    with pytest.raises(AssertionError):
        b[1][3] = [333, 33]
        deep_compare(a, b)

    a = [{2: 22, 5: '55'}, [33, 333, '3'], {6: [6], 4: 44}]
    b = [{5: '55', 2: 22}, [33, 333, '3'], {4: 44, 6: [6]}]
    deep_compare(a, b)
    with pytest.raises(AssertionError):
        b[1][2] = 3
        deep_compare(a, b)

    a = [
           {'ip_addr': '0.0.0.0', 'process_name': 'qpidd', 'port': '5672'},
           {'ip_addr': '127.0.0.1', 'process_name': 'mongod', 'port': '27017'},
           {'ip_addr': '127.0.0.1', 'process_name': 'Passenger Rac', 'port': '53644'},
           {'ip_addr': '0.0.0.0', 'process_name': 'qdrouterd', 'port': '5646'},
    ]
    b = [
           {'ip_addr': '0.0.0.0', 'port': '5672', 'process_name': 'qpidd'},
           {'ip_addr': '127.0.0.1', 'port': '27017', 'process_name': 'mongod'},
           {'ip_addr': '127.0.0.1', 'port': '53644', 'process_name': 'Passenger Rac'},
           {'ip_addr': '0.0.0.0', 'port': '5646', 'process_name': 'qdrouterd'},
    ]
    deep_compare(a, b)


def test_deep_nest_dict_list():
    a = {'2': [22, 222], 3: [33, 333]}
    b = {3: [33, 333], '2': [22, 222]}
    deep_compare(a, b)
    with pytest.raises(AssertionError):
        b['2'] = [222, 22]
        deep_compare(a, b)

    a = {3: [33, [3, '33', 333]], 2: (22, 222)}
    b = {2: (22, 222), 3: [33, [3, '33', 333]]}
    with pytest.raises(AssertionError):
        b[3] = [33, ['33', 333, 3]]
        deep_compare(a, b)

    a = {3: [33, [set([4, 3]), '33', 333]], 2: (22, 222)}
    b = {2: (22, 222), 3: [33, [set([4, 3]), '33', 333]]}
    with pytest.raises(AssertionError):
        b[3][1] = set([3, 4])
        deep_compare(a, b)


def test_deep_compare_special():
    """
    Test the special format of the `expected`
    """
    a = {'type': 'skip', 'reason': 'MISSING_REQUIREMENTS', 'details': 'test1'}
    b1 = (None, 'test1')
    b2 = [None, ['test1']]
    deep_compare(a, b1)
    deep_compare(a, b2)

    with pytest.raises(AssertionError):
        a = {'type': 'skip', 'reason': 'NO_MISSING', 'details': 'test1'}
        deep_compare(a, b1)

    with pytest.raises(AssertionError):
        a = {'type': 'skip', 'reason': 'MISSING_REQUIREMENTS', 'details': 'test0'}
        deep_compare(a, b1)


def test_case_variants():
    filter_list = ['Ciphers', 'MACs', 'UsePAM', 'MaxAuthTries', 'nt pipe support',
                   'A-Dash-SEPARATED-tESt-tEST-tesT-test-ExAMPle']
    expanded_list = ['Ciphers', 'ciphers', 'CIPHERS',
                     'MACs', 'Macs', 'macs', 'MACS',
                     'UsePAM', 'UsePam', 'usepam', 'USEPAM', 'Usepam',
                     'MaxAuthTries', 'maxauthtries', 'MAXAUTHTRIES', 'Maxauthtries',
                     'nt pipe support', 'NT PIPE SUPPORT', 'Nt Pipe Support',
                     'A-Dash-SEPARATED-tESt-tEST-tesT-test-ExAMPle',
                     'A-Dash-Separated-tEst-tEst-tesT-test-ExAmple',
                     'a-dash-separated-test-test-test-test-example',
                     'A-DASH-SEPARATED-TEST-TEST-TEST-TEST-EXAMPLE',
                     'A-Dash-Separated-Test-Test-Test-Test-Example']
    assert case_variants(*filter_list) == expanded_list

    assert case_variants('hosts:') == ['hosts:', 'HOSTS:', 'Hosts:']


@pytest.mark.skipif(sys.version_info < (2, 7), reason='Code with PYTEST_CURRENT_TEST is incompatible with py26')
def test_deprecated():
    def normal_fn():
        return 1

    def deprecated_fn():
        deprecated(deprecated_fn, "really don't use this")
        return 3

    assert normal_fn() == 1

    # For all remaining tests, cause the warnings to always be caught
    warnings.simplefilter('always')

    with warnings.catch_warnings(record=True) as w:
        assert deprecated_fn() == 3

        assert len(w) == 1
        assert issubclass(w[0].category, DeprecationWarning)
        assert "really don't use this" in str(w[0].message)

    # Reset warnings to default
    warnings.resetwarnings()



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import pci_rport_target_disk_paths
from insights.parsers.pci_rport_target_disk_paths import PciRportTargetDiskPaths as PCIPaths
from insights.tests import context_wrap


INPUT_GOOD = """/sys/devices/pci0000:00/0000:00:01.0/0000:04:00.6/host1/rport-1:0-1/target1:0:0/1:0:0:0/block/sdb/stat
/sys/devices/pci0000:00/0000:00:01.0/0000:04:00.7/host2/rport-2:0-2/target2:0:0/2:0:0:0/block/sdc/stat
/sys/devices/pci0000:00/0000:00:02.2/0000:02:00.0/host0/target0:1:0/0:1:0:0/block/sda/stat
""".strip()


INPUT_BAD = """
/sys/devices/pci0000:00/block/sda/stat
/sys/devices/pci0000:00/block/sda/device
/sys/devices/pci0000:00/block/sda/events
/sys/devices/pci0000:00/block/sda/subsystem
/sys/devices/pci0000:00/block/sda/ext_range
/sys/devices/pci0000:00/block/sda/slaves
/sys/devices/pci0000:00/block/sda/uevent
/sys/devices/pci0000:00/block/sda/events_poll_msecs
/sys/devices/pci0000:00/block/sda/alignment_offset
/sys/devices/pci0000:00/block/sda/holders
/sys/devices/pci0000:00/block/sda/inflight
/sys/devices/pci0000:00/block/sda/removable
/sys/devices/pci0000:00/block/sda/capability
/sys/devices/pci0000:00/block/sda/events_async
""".strip()


def test_good():
    pi = PCIPaths(context_wrap(INPUT_GOOD))
    assert pi.path_list[0]['pci_id'] == '0000:04:00.6'
    assert pi.path_list[0]['target'] == 'target1:0:0'

    assert pi.path_list[0]['rport'] == 'rport-1:0-1'
    assert pi.path_list[0]['devnode'] == 'sdb'

    assert pi.pci_id == ['0000:02:00.0', '0000:04:00.6', '0000:04:00.7']
    assert pi.devnode == ['sda', 'sdb', 'sdc']

    assert pi.host == ['host0', 'host1', 'host2']
    assert pi.target == ['target0:1:0', 'target1:0:0', 'target2:0:0']

    assert pi.rport == ['rport-1:0-1', 'rport-2:0-2']
    assert pi.host_channel_id_lun == ['0:1:0:0', '1:0:0:0', '2:0:0:0']


def test_status_documentation():
    """
    Here we test the examples in the documentation automatically using doctest
    . We set up an environment which is similar to what a rule writer might
    see - a '/usr/bin/find /sys/devices/pci0000:00 -mindepth 8 -maxdepth 8' command
    output that has been passed in as a parameter to the rule declaration.
    """
    env = {'pd': PCIPaths(context_wrap(INPUT_GOOD))}
    failed, total = doctest.testmod(pci_rport_target_disk_paths, globs=env)
    assert failed == 0


def test_status_exp():
    """
    Here test the examples cause expections
    """
    with pytest.raises(SkipComponent) as sc1:
        PCIPaths(context_wrap(''))
    assert "Input content is empty" in str(sc1)


def test_status_exp2():
    """
    Here test the examples cause expections
    """
    with pytest.raises(ParseException) as sc1:
        PCIPaths(context_wrap(INPUT_BAD))
    assert "No useful data parsed in line" in str(sc1)



# -*- coding: UTF-8 -*-
import pytest

from insights.core import TextFileOutput
from insights.tests import context_wrap


class FakeMessagesClass(TextFileOutput):
    def listget(self, s):
        # Silly function to split lines containing s.
        for line in self.lines:
            if s in line:
                yield s.split(None, 6)


MESSAGES = """
Mar 27 03:18:15 system rsyslogd: [origin software="rsyslogd" swVersion="5.8.10" x-pid="1870" x-info="http://www.rsyslog.com"] first
Mar 27 03:18:16 system rsyslogd-2177: imuxsock lost 141 messages from pid 55082 due to rate-limiting
Mar 27 03:18:19 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:18:21 system pulp: pulp.server.db.connection:INFO: Attempting Database connection with seeds = localhost:27017
Mar 27 03:18:21 system pulp: pulp.server.db.connection:INFO: Connection Arguments: {'max_pool_size': 10}
Mar 27 03:18:21 system pulp: pulp.server.db.connection:INFO: Database connection established with: seeds = localhost:27017, name = pulp_database
Mar 27 03:18:22 system rsyslogd-2177: imuxsock lost 145 messages from pid 55082 due to rate-limiting
Mar 27 03:18:24 system puppet-master[48226]: Setting manifest is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations
Mar 27 03:18:24 system puppet-master[48226]:    (at /usr/lib/ruby/site_ruby/1.8/puppet/settings.rb:1095:in `issue_deprecations')
Mar 27 03:18:24 system puppet-master[48226]: Setting modulepath is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations
Mar 27 03:18:24 system puppet-master[48226]:    (at /usr/lib/ruby/site_ruby/1.8/puppet/settings.rb:1095:in `issue_deprecations')
Mar 27 03:18:24 system puppet-master[48226]: Setting config_version is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations
Mar 27 03:18:24 system puppet-master[48226]:    (at /usr/lib/ruby/site_ruby/1.8/puppet/settings.rb:1095:in `issue_deprecations')
Mar 27 03:18:25 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:39:43 system pulp: pulp.server.webservices.middleware.exception:ERROR:   File "/usr/lib/python2.6/site-packages/web/application.py", line 230, in handle
     return self._delegate(fn, self.fvars, args)
   File "/usr/lib/python2.6/site-packages/web/application.py", line 405, in _delegate
     return handle_class(f)
   File "/usr/lib/python2.6/site-packages/web/application.py", line 396, in handle_class
     return tocall(*args)
   File "/usr/lib/python2.6/site-packages/pulp/server/webservices/controllers/decorators.py", line 227, in _auth_decorator
     value = method(self, *args, **kwargs)
   File "/usr/lib/python2.6/site-packages/pulp/server/webservices/controllers/consumers.py", line 503, in GET
     profile = manager.get_profile(consumer_id, content_type)
   File "/usr/lib/python2.6/site-packages/pulp/server/managers/consumer/profile.py", line 120, in get_profile
     raise MissingResource(profile_id=profile_id)
MissingResource: Missing resource(s): profile_id={'content_type': u'rpm', 'consumer_id': u'1786cd7f-2ab2-4212-9798-c0a454e97900'}
Mar 27 03:39:43 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:39:46 system rsyslogd-2177: imuxsock lost 165 messages from pid 55082 due to rate-limiting
Mar 27 03:39:49 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:49:10 system pulp: pulp.server.webservices.middleware.exception:ERROR: Missing resource(s): profile_id={'content_type': u'rpm', 'consumer_id': u'79d5aed1-5631-4f40-b970-585ee974eb87'}
"""


def test_lines_scanners():
    # Messages that are present can be kept
    FakeMessagesClass.keep_scan('puppet_master_logs', ' puppet-master')
    # Messages that are absent still turn up as an empty list
    FakeMessagesClass.keep_scan('kernel_logs', ' kernel')
    # Token scan of something that's present
    FakeMessagesClass.token_scan('middleware_exception_present', 'pulp.server.webservices.middleware.exception')
    # Token scan of something that's absent
    FakeMessagesClass.token_scan('cron_present', 'CRONTAB')

    # Check that duplicate scanners raise a value exception
    with pytest.raises(ValueError) as exc:
        FakeMessagesClass.keep_scan('kernel_logs', ' kernel')
    assert "'kernel_logs' is already a registered scanner key" in str(exc)

    def count_lost_lines(self):
        imuxsock_lines = 0
        lost_lines = 0
        for line in self.lines:
            if 'imuxsock lost' in line:
                parts = line.split(None)
                if parts[7].isdigit():
                    imuxsock_lines += 1
                    lost_lines += int(parts[7])
        return "lost {msgs} lines in {lines} lines".format(msgs=lost_lines, lines=imuxsock_lines)
    FakeMessagesClass.scan('lost_lines', count_lost_lines)

    ctx = context_wrap(MESSAGES, path='/var/log/messages')
    log = FakeMessagesClass(ctx)
    assert hasattr(log, 'puppet_master_logs')
    assert hasattr(log, 'kernel_logs')
    assert hasattr(log, 'middleware_exception_present')
    assert hasattr(log, 'cron_present')
    assert hasattr(log, 'lost_lines')

    assert len(log.puppet_master_logs) == 6
    assert log.puppet_master_logs[0]['raw_line'] == 'Mar 27 03:18:24 system puppet-master[48226]: Setting manifest is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations'
    assert log.kernel_logs == []
    assert log.middleware_exception_present
    assert not log.cron_present
    assert log.lost_lines == 'lost 451 lines in 3 lines'


def test_lines_scanners_list():
    # Messages that are present can be kept
    FakeMessagesClass.keep_scan('puppet_master_manifest_logs', ['puppet-master', 'manifest'])
    FakeMessagesClass.keep_scan('puppet_master_first', ['puppet-master', 'first'])
    # Messages that are present can be kept for any of the list
    FakeMessagesClass.keep_scan('puppet_master_first_any', ['puppet-master', 'first'], check=any)
    FakeMessagesClass.keep_scan('puppet_master_first_any_3', ['puppet-master', 'first'], num=3, check=any)
    # Token scan of something that's absent
    FakeMessagesClass.token_scan('error_missing', ['ERROR', 'Missing'])
    # Token scan of something that's absent
    FakeMessagesClass.token_scan('error_info', ['ERROR', 'info'])
    # Token scan of something that's absent for any of the list
    FakeMessagesClass.token_scan('error_info_any', ['ERROR', 'info'], check=any)
    # Get the last line
    FakeMessagesClass.last_scan('puppet_master_manifest_last', ['puppet-master', 'manifest'])
    # Get the last line for any of the list
    FakeMessagesClass.last_scan('puppet_master_first_last_any', ['puppet-master', 'first'], check=any)

    ctx = context_wrap(MESSAGES, path='/var/log/messages')
    log = FakeMessagesClass(ctx)

    assert hasattr(log, 'puppet_master_manifest_logs')
    assert len(log.puppet_master_manifest_logs) == 1
    assert hasattr(log, 'error_missing')
    assert log.error_missing

    assert hasattr(log, 'puppet_master_first')
    assert len(log.puppet_master_first) == 0
    assert hasattr(log, 'error_info')
    assert log.error_info is False

    assert hasattr(log, 'puppet_master_first_any')
    assert len(log.puppet_master_first_any) == 7
    assert hasattr(log, 'puppet_master_first_any_3')
    assert len(log.puppet_master_first_any_3) == 3
    assert hasattr(log, 'error_info_any')
    assert log.error_info_any is True

    assert hasattr(log, 'puppet_master_manifest_last')
    assert 'puppet-master' in log.puppet_master_manifest_last['raw_line']
    assert 'manifest' in log.puppet_master_manifest_last['raw_line']
    assert hasattr(log, 'puppet_master_first_last_any')
    assert 'puppet-master' in log.puppet_master_first_last_any['raw_line']


MESSAGES_ROLLOVER_YEAR = """
Dec 31 21:43:00 duradm13 [CMA]: Logger failed to open catalog file
Dec 31 22:03:05 duradm13 xinetd[21465]: START: bgssd pid=28021 from=10.20.40.7
Dec 31 22:03:05 duradm13 xinetd[21465]: EXIT: bgssd status=0 pid=28021 duration=0(sec)
Dec 31 23:03:07 duradm13 xinetd[21465]: START: bgssd pid=31307 from=10.20.40.7
Dec 31 23:03:07 duradm13 xinetd[21465]: EXIT: bgssd status=0 pid=31307 duration=0(sec)
Dec 31 23:07:00 duradm13 [CMA]: Logger failed to open catalog file
Jan  1 00:00:00 duradm13 [CMA]: Logger failed to open catalog file
Jan  1 00:03:09 duradm13 xinetd[21465]: START: bgssd pid=2203 from=10.20.40.7
Jan  1 00:03:09 duradm13 xinetd[21465]: EXIT: bgssd status=0 pid=2203 duration=0(sec)
Jan  1 00:11:45 duradm13 xinetd[21465]: START: vnetd pid=2670 from=10.20.40.36
Jan  1 00:11:47 duradm13 xinetd[21465]: START: vnetd pid=2671 from=10.20.40.36
Jan  1 00:11:48 duradm13 xinetd[21465]: EXIT: vnetd status=0 pid=2671 duration=1(sec)
Jan  1 01:00:08 duradm13 xinetd[21465]: START: nrpe pid=6189 from=10.20.40.240
Jan  1 01:00:08 duradm13 xinetd[21465]: EXIT: nrpe status=0 pid=6189 duration=0(sec)
Jan  1 01:00:27 duradm13 xinetd[21465]: START: nrpe pid=6207 from=10.20.40.240
Jan  1 01:00:27 duradm13 xinetd[21465]: EXIT: nrpe status=0 pid=6207 duration=0(sec)
Jan  1 01:00:29 duradm13 xinetd[21465]: START: nrpe pid=6210 from=10.20.40.240
Jan  1 01:00:29 duradm13 xinetd[21465]: EXIT: nrpe status=0 pid=6210 duration=0(sec)
""".strip()


def test_lines_2():
    ctx = context_wrap(MESSAGES_ROLLOVER_YEAR, path='/var/log/messages')
    log = FakeMessagesClass(ctx)
    assert len(log.lines) == 18



from insights.parsers import ros_config
from insights.tests import context_wrap
import doctest


ROS_CONFIG_INPUT = """
log mandatory on default {
    mem.util.used
    mem.physmem
    kernel.all.cpu.user
    kernel.all.cpu.sys
    kernel.all.cpu.nice
    kernel.all.cpu.steal
    kernel.all.cpu.idle
    kernel.all.cpu.wait.total
    disk.all.total
    mem.util.cached
    mem.util.bufmem
    mem.util.free
}
[access]
disallow .* : all;
disallow :* : all;
allow local:* : enquire;
"""


def test_ros_config():
    ros_input = ros_config.RosConfig(context_wrap(ROS_CONFIG_INPUT))
    assert ros_input.data is not None
    assert ros_input.specs[0].get('state') == 'mandatory on'
    assert ros_input.specs[0].get('metrics') == {'mem.util.used': [],
                                                  'mem.physmem': [],
                                                  'kernel.all.cpu.user': [],
                                                  'kernel.all.cpu.sys': [],
                                                  'kernel.all.cpu.nice': [],
                                                  'kernel.all.cpu.steal': [],
                                                  'kernel.all.cpu.idle': [],
                                                  'kernel.all.cpu.wait.total': [],
                                                  'disk.all.total': [],
                                                  'mem.util.cached': [],
                                                  'mem.util.bufmem': [],
                                                  'mem.util.free': []}
    assert ros_input.specs[0].get('logging_interval') == 'default'
    assert ros_input.rules == [
                                {
                                    'allow_disallow': 'disallow',
                                    'hostlist': ['.*'],
                                    'operationlist': ['all']
                                },
                                {
                                    'allow_disallow': 'disallow',
                                    'hostlist': [':*'],
                                    'operationlist': ['all']
                                },
                                {
                                    'allow_disallow': 'allow',
                                    'hostlist': ['local:*'],
                                    'operationlist': ['enquire']
                                }
                               ]


def test_ros_config_documentation():
    env = {
        'ros_input': ros_config.RosConfig(context_wrap(ROS_CONFIG_INPUT, path='/var/lib/pcp/config/pmlogger/config.ros')),
    }
    failed, total = doctest.testmod(ros_config, globs=env)
    assert failed == 0



"""
IBM Proc Files
==============

Parsers for parsing the following proc files of IBM system:

IBMPpcLparCfg - file ``/proc/powerpc/lparcfg``
----------------------------------------------

IBMFirmwareLevel - file ``/proc/device-tree/openprom/ibm,fw-vernum_encoded``
----------------------------------------------------------------------------
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ibm_lparcfg)
class IBMPpcLparCfg(Parser, dict):
    """
    Class for parsing the ``/proc/powerpc/lparcfg`` file to a dictionary.

    Typical content looks like::

        serial_number=IBM,123456789
        system_type=IBM,8247-22L

    Examples:
        >>> ibm_mtm['system_type']
        '8247-22L'

    Raises:
        SkipComponent: If nothing need to parse
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty output.")

        for line in content:
            key, value = [l.strip() for l in line.split('=', 1)]
            _, _, value = value.partition(',')
            self[key] = value.strip()

        if len(self) == 0:
            raise SkipComponent("Nothing to parse.")


@parser(Specs.ibm_fw_vernum_encoded)
class IBMFirmwareLevel(Parser):
    """
    Class for parsing the ``/proc/device-tree/openprom/ibm,fw-vernum_encoded``
    file.

    Typical content looks like::

        FW950.30 (VL950_092)\x00

    Attributes:
        raw (str): The RAW content of the `ibm,fw-vernum_encoded` file.
        firmware_level (str): The firmware level required by FLRT.

    Examples:
        >>> ibm_fwl.firmware_level
        'VL950_092'

    Raises:
        SkipComponent: If nothing need to parse
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty output.")

        self.raw = content[0]

        if "(" not in self.raw or ")" not in self.raw:
            raise SkipComponent("Nothing to parse.")

        self.firmware_level = self.raw[self.raw.index('(') + 1:self.raw.index(')')]



"""
This module contains the following parsers:

VDSMConfIni - file ``/etc/vdsm/vdsm.conf``
------------------------------------------
VDSMLoggerConf - file ``/etc/vdsm/logger.conf``
-----------------------------------------------
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.vdsm_conf)
class VDSMConfIni(IniConfigFile):
    """This class parses the ``/etc/vdsm/vdsm.conf`` file using the
    ``IniConfigFile`` base parser.

    Sample configuration file::

        [vars]
        ssl = true
        cpu_affinity = 1

        [addresses]
        management_port = 54321
        qq = 345

    Examples:
        >>> 'vars' in conf
        True
        >>> conf.get('addresses', 'qq') == '345'
        True
        >>> conf.getboolean('vars', 'ssl')
        True
        >>> conf.getint('addresses', 'management_port')
        54321
    """
    pass


@parser(Specs.vdsm_logger_conf)
class VDSMLoggerConf(IniConfigFile):
    """This class parses the ``/etc/vdsm/logger.conf`` file using the
    ``IniConfigFile`` base parser.

    Sample configuration file::

        [loggers]
        keys=root,vds,storage,virt,ovirt_hosted_engine_ha,ovirt_hosted_engine_ha_config,IOProcess,devel

        [formatter_long]
        format: %(asctime)s %(levelname)-5s (%(threadName)s) [%(name)s] %(message)s (%(module)s:%(lineno)d)
        class: vdsm.logUtils.TimezoneFormatter

        [logger_root]
        level=DEBUG
        handlers=syslog,logfile
        propagate=0

        [formatters]
        keys=long,simple,none,sysform

        [logger_ovirt_hosted_engine_ha]
        level=DEBUG
        handlers=
        qualname=ovirt_hosted_engine_ha
        propagate=1

        [formatter_sysform]
        format= vdsm %(name)s %(levelname)s %(message)s
        datefmt=

    Examples:
        >>> len(vdsm_logger_conf.sections())
        18
        >>> vdsm_logger_conf.has_option('formatter_long', 'class')
        True
        >>> vdsm_logger_conf.has_option('loggers', 'keys')
        True
        >>> vdsm_logger_conf.getboolean('logger_root', 'propagate')
        False
        >>> vdsm_logger_conf.items('loggers') == {'keys': 'root,vds,storage,virt,ovirt_hosted_engine_ha,ovirt_hosted_engine_ha_config,IOProcess,devel'}
        True
        >>> vdsm_logger_conf.get('logger_ovirt_hosted_engine_ha', 'level') == 'DEBUG'
        True
        >>> vdsm_logger_conf.get('formatter_sysform', 'datefmt') == ''
        True
    """
    pass



from insights.parsers import dcbtool_gc_dcb
from insights.tests import context_wrap


DCBTOOL_GC_OUTPUT = """

    Command:    Get Config
    Feature:    DCB State
    Port:       eth0
    Status:     Off
    DCBX Version: FORCED CIN

"""

DCBTOOL_GC_DCB_FAILED = """
connect: Connection refused
Failed to connect to lldpad - clif_open: Connection refused
"""


def test_dcbtool_gc():
    result = dcbtool_gc_dcb.Dcbtool(context_wrap(DCBTOOL_GC_OUTPUT))
    assert len(result.data) == 5
    assert result["command"] == "Get Config"
    assert result["feature"] == "DCB State"
    assert result["port"] == "eth0"
    assert result["status"] == "Off"
    assert result["dcbx_version"] == "FORCED CIN"

    assert not result.is_on

    result = dcbtool_gc_dcb.Dcbtool(context_wrap(DCBTOOL_GC_DCB_FAILED))
    assert len(result.data) == 0



"""
SetupNamedChroot - file ``/usr/libexec/setup-named-chroot.sh``
==============================================================

This module provides class ``SetupNamedChroot`` for parsing the output of file
``/usr/libexec/setup-named-chroot.sh``.
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs

add_filter(Specs.setup_named_chroot, ["ROOTDIR_MOUNT", "/"])


@parser(Specs.setup_named_chroot)
class SetupNamedChroot(Parser, dict):
    """
    Class for parsing the `/usr/libexec/setup-named-chroot.sh` file.

    Typical content of the filtered file is::

        #!/bin/bash
        # it MUST be listed last. (/var/named contains /var/named/chroot)
        ROOTDIR_MOUNT='/etc/localtime /etc/named /etc/pki/dnssec-keys /etc/named.root.key /etc/named.conf
        /etc/named.dnssec.keys /etc/named.rfc1912.zones /etc/rndc.conf /etc/rndc.key /usr/lib64/bind
        /usr/lib/bind /etc/named.iscdlv.key /run/named /var/named /etc/protocols /etc/services'
            for all in $ROOTDIR_MOUNT; do
            for all in $ROOTDIR_MOUNT; do,
            # Check if file is mount target. Do not use /proc/mounts because detecting

    Attributes:
        raw (list): A list of all the active lines present

    Raises:
        SkipComponent: When the file is empty or when the input content is not
                       empty but there is no useful parsed data

    Examples:
        >>> len(snc)
        2
        >>> snc['ROOTDIR_MOUNT']
        ['/etc/localtime', '/etc/named', '/etc/pki/dnssec-keys', '/etc/named.root.key', '/etc/named.conf', '/etc/named.dnssec.keys', '/etc/named.rfc1912.zones', '/etc/rndc.conf', '/etc/rndc.key', '/etc/named.iscdlv.key', '/etc/protocols', '/etc/services', '/usr/lib64/bind', '/usr/lib/bind', '/run/named', '/var/named']
    """

    def parse_content(self, content):
        # No content found or file is empty
        if not content:
            raise SkipComponent("Empty file")

        data = {}
        self.raw = []
        key, value = "", ""
        for line in get_active_lines(content):
            self.raw.append(line.strip())
            if "=" in line:
                key, value = line.split("=", 1)
            elif key:
                if value.endswith("'") or value.endswith("\""):
                    data[key] = value[1:-1].split()
                    key, value = "", ""
                value = value + " " + line

        # No useful parsed data
        if not data:
            raise SkipComponent("Input content is not empty but there is no useful parsed data.")

        self.update(data)



"""
JBoss version
=============
Provide information about the versions of all running Jboss on a system.
"""
import json

from collections import namedtuple
from insights import Parser, parser
from insights.specs import Specs

# define namedtuple to store the property of version
_VersionNameTuple = namedtuple("_VersionNameTuple", ["file_path", "product", "version", "code_name", "major", "minor", "release"])


def _get_version_tuple(version_line, i_file_path):
    """
    Perform the version line parsing, returning a nametuple of the values of one jboss version
    """
    product, _, version_name = [v.strip() for v in version_line.partition("- Version")]
    if " GA" in version_name:
        # handle Red Hat JBoss Web Server - Version 5.6 GA
        version = version_name.split(' GA')[0]
        code_name = "GA"
        updated_version = version + ".0"
        major, minor, release = updated_version.split('.')[0:3]
    else:
        # add empty code name for Red Hat Data Grid - Version 7.3.0
        version_name = version_name.strip() + "."
        major, minor, release, code_name = version_name.split(".")[0:4]
        version = '.'.join([major, minor, release])

    return _VersionNameTuple(i_file_path, product, version, code_name, int(major), int(minor), int(release))


@parser(Specs.jboss_version)
class JbossVersion(Parser):
    """
    This class is to access to file ``$JBOSS_HOME/version.txt``

    Typical content of file ``$JBOSS_HOME/version.txt`` is::

        Red Hat JBoss Enterprise Application Platform - Version 6.4.3.GA

    This class parses the file content and stores data in the dict ``self.parsed``.
    The version info can also be got via ``obj.major`` and ``obj.minor``, etc.

    Examples:
        >>> jboss_version.file_path
        '/home/test/jboss/jboss-eap-6.4/version.txt'
        >>> jboss_version.raw
        'Red Hat JBoss Enterprise Application Platform - Version 6.4.3.GA'
        >>> jboss_version.major
        6
        >>> jboss_version.minor
        4
        >>> jboss_version.release
        3
        >>> jboss_version.version
        '6.4.3'
        >>> jboss_version.code_name
        'GA'
    """

    def parse_content(self, content):
        self.raw = content[0]
        self._parsed = _get_version_tuple(content[0], self.file_path)

    @property
    def product(self):
        """string: the version of this running JBoss progress."""
        return self._parsed.product

    @property
    def version(self):
        """string: the version of this running JBoss progress."""
        return self._parsed.version

    @property
    def major(self):
        """int: the major version of this running JBoss progress."""
        return self._parsed.major

    @property
    def minor(self):
        """int: the minor version of this running JBoss progress."""
        return self._parsed.minor

    @property
    def release(self):
        """int: release of this running JBoss progress."""
        return self._parsed.release

    @property
    def code_name(self):
        """string: code name of this running JBoss progress."""
        return self._parsed.code_name


@parser(Specs.jboss_runtime_versions)
class JbossRuntimeVersions(Parser, list):
    """
     This class is to access to file ``data/insights_commands/jboss_versions``

     Typical content of file ``data/insights_commands/jboss_versions`` is::

         {"/opt/jboss-datagrid-7.3.0-server": "Red Hat Data Grid - Version 7.3.0"}

     This class parses the file content and stores data in the list.

     Examples:
         >>> len(all_jboss_versions)
         1
         >>> all_jboss_versions[0].major
         7
         >>> all_jboss_versions[0].minor
         3
         >>> all_jboss_versions[0].release
         0
     """

    def parse_content(self, content):
        jboss_version_dict = json.loads(' '.join(content))
        for j_path, version_content in jboss_version_dict.items():
            lines = version_content.strip().splitlines()
            self.append(_get_version_tuple(lines[0], j_path))
